import onkopus
from onkopus.processing.parallel_gene_requests import parallel_requests



def identify_genes_from_request(gene_names, genome_version=None):
    """
    Generates a gene JSON object from a text query

    :param gene_names:
    :param genome_version:
    :return:
    """
    if len(gene_names)>0:
        annotated_data = {}

        for gene in gene_names:
            annotated_data[gene] = {}
            annotated_data[gene]["variant_data"] = {}
            annotated_data[gene]["variant_data"]["mutation_type"] = "gene"

        #annotated_data = annotate_gene_request(annotated_data, genome_version=genome_version)
        return annotated_data
    else:
        return {}


def annotate_genes(
        annotated_data,
        genome_version="hg38",
        tumor_type=None,
        request_id=None,include_clinical_data=True,
                  include_gene=True,
                  include_acmg=True):
    """
    Annotates a gene with associated Onkopus modules

    :param annotated_data:
    :param gene:
    :param genome_version:
    :return:
    """
    # get genes from biomarker data
    gene_data = {}
    for var in annotated_data.keys():
        mut_type = ""

        if "variant_data" in annotated_data[var]:
            if "mutation_type" in annotated_data[var]["variant_data"]:
                mut_type = annotated_data[var]["variant_data"]["mutation_type"]
        elif "mutation_type" in annotated_data[var]:
            mut_type = annotated_data[var]["mutation_type"]
        if mut_type == "gene":
            gene_data[var] = annotated_data[var]

    if len(list(gene_data.keys())) > 0:
        gene_data = parallel_requests(gene_data, genome_version=genome_version)
        if include_clinical_data is True:
            gene_data = onkopus.onkopus_clients.AggregatorClient(genome_version=genome_version).process_data(gene_data)

    for var in gene_data.keys():
        annotated_data[var] = gene_data[var]

        annotated_data[var]["UTA_Adapter"] = { "gene_name": var }

    #return annotated_data
    return gene_data



import os

os.environ["ONKOPUS_MODULE_SERVER2"] = "134.76.19.66"
os.environ["ONKOPUS_MODULE_PROTOCOL2"] = "http"

import unittest
import adagenes as ag
import onkopus as op


class DSTestCase(unittest.TestCase):

    def test_ds_client(self):
        genome_version = 'hg38'

        data = {"chr1:100000-200000_DEL": {}, "chr2:200000-1900000_DUP": {}}
        bframe = ag.BiomarkerFrame(data)
        print(bframe.data)

        #variant_data = op.GENCODECNAClient(genome_version).process_data(bframe.data)
        variant_data = op.CNASphereGenesClient(genome_version).process_data(bframe.data)
        print(variant_data)

        variant_data = op.DosageSensitivityClient(genome_version).process_data(variant_data)

        print("keys ", variant_data)
        print(variant_data["chr2:200000-1900000_DUP"]["dosage_sensitivity"])
        self.assertEqual(variant_data["chr2:200000-1900000_DUP"]["dosage_sensitivity"]['MYT1L']["chrom"], "chr2", "")




import unittest, os
import onkopus as op


class BiomarkerRecognitionTestCase(unittest.TestCase):

    def test_silent_mutation_recognition(self):
        __location__ = os.path.realpath(
            os.path.join(os.getcwd(), os.path.dirname(__file__)))
        infile = __location__ + "/../test_files/somaticMutations.hg38.ln200.vcf"
        # bframe = op.read_file(__location__ + "/../test_files/variants_grch38_20240206_Pathogenic_80.tsv")
        bframe = op.read_file(infile)
        genome_version="hg38"

        bframe.data = op.UTAAdapterClient(genome_version="hg38").process_data(bframe.data)
        bframe.data = op.CCSGeneToGenomicClient(genome_version=genome_version).process_data(
            bframe.data, data_type="g")

        for var in bframe.data.keys():
            #vartype = bframe.data[var]["type"]
            vstr = ""
            #print(vartype)
            if "UTA_Adapter" in bframe.data[var].keys():
                if "variant_exchange" in bframe.data[var]["UTA_Adapter"].keys():
                    vstr += var + ": "
                    vstr += bframe.data[var]["UTA_Adapter"]["variant_exchange"]
                    vstr += ":" + bframe.data[var]["mutation_type"]
                    print(vstr)

        self.assertIn("chr18:7888143G>A", list(bframe.data.keys()), f"chr18:7888143G>A is not in the list")
        self.assertEqual(bframe.data["chr18:7888143G>A"]["UTA_Adapter"]["variant_exchange"],"E78=","")

    def test_silent_mutation_annotation(self):
        data = {"chr18:7888143G>A": {}}

        data = op.annotate_variant_data(data)
        print(data)

        self.assertEqual(["chr18:7888143G>A"],list(data.keys()))







import unittest, copy, os
import onkopus as op
import adagenes as ag

class MolFeatAnnotationTestCase(unittest.TestCase):

    def test_molfeat_hg19(self):
        data = {"chr7:140453136A>T":{}}
        genome_version="hg19"
        variant_data = ag.LiftoverAnnotationClient(genome_version=genome_version).process_data(data)
        print(variant_data)
        variant_data = op.UTAAdapterClient(genome_version=genome_version).process_data(variant_data)
        variant_data = op.MolecularFeaturesClient(
            genome_version=genome_version).process_data(variant_data)
        self.assertEqual(variant_data["chr7:140453136A>T"]["molecular_features"]["aromaticity_alt"], 0, "")
        self.assertEqual(variant_data["chr7:140453136A>T"]["molecular_features"]["aromaticity_diff"], 0, "")

    def test_molfeat_client(self):
        #genome_version = 'hg19'
        genome_version = 'hg38'

        #data = {"chr17:7681744T>C": {}, "chr10:8115913C>T": {}}
        data = {"chr7:140753336A>T":{}}

        variant_data = op.UTAAdapterClient(genome_version=genome_version).process_data(data)
        variant_data = op.MolecularFeaturesClient(
            genome_version=genome_version).process_data(variant_data)

        #print("Response ",variant_data)
        self.assertEqual(variant_data["chr7:140753336A>T"]["molecular_features"]["aromaticity_alt"],0,"")

    def test_molfeat_export(self):
        genome_version = 'hg38'
        __location__ = os.path.realpath(
            os.path.join(os.getcwd(), os.path.dirname(__file__)))
        infile = __location__ + "/../../test_files/somaticMutations.l520.protein.vcf"
        outfile = __location__ + "/../../test_files/somaticMutations.l520.protein.molfeat.vcf"

        data = {"chr7:140753336A>T": {}}

        #variant_data = op.UTAAdapterClient(genome_version=genome_version).process_data(data)
        magic_obj = op.MolecularFeaturesClient(
            genome_version=genome_version)
        #op.write_file(outfile, variant_data)

        ag.process_file(infile, outfile, magic_obj=magic_obj)

        file = open(outfile)
        contents = file.read()[0:500]
        print(contents)
        contents_expected = ('chr7\t21744592\t0\tA\tAG\t0\t0\t0;POS_hg19=21784210;POS_hg38=21744592;;\n'
 'chr10\t8073950\t0\tC\tT\t0\t0\t'
 '0;POS_hg19=8115913;POS_hg38=8073950;UTA_Adapter_gene_name=GATA3;UTA_Adapter_variant_exchange=T421M;UTA_Adapter_input_data=chr10:g.8073950C>T;UTA_Adapter_transcript=NM_001002295.2;UTA_Adapter_variant=NP_001002295.1:p.(Thr421Met);UTA_Adapter_variant_exchange_long=Thr421Met;molecular_features_molecular_weight_alt=149.2113;molecular_features_molecular_weight_ref=119.1192;molecular_features_molecular_weight_diff=')
        self.assertEqual(contents, contents_expected, "")
        file.close()

    def test_molfeat_export_writefile(self):
        # genome_version = 'hg19'
        genome_version = 'hg38'
        __location__ = os.path.realpath(
            os.path.join(os.getcwd(), os.path.dirname(__file__)))
        infile = __location__ + "/../../test_files/somaticMutations.l520.protein.vcf"
        outfile = __location__ + "/../../test_files/molfeat.export_test.vcf"

        # data = {"chr17:7681744T>C": {}, "chr10:8115913C>T": {}}
        data = {"chr7:140753336A>T": {}}

        variant_data = op.UTAAdapterClient(genome_version=genome_version).process_data(data)
        variant_data = op.MolecularFeaturesClient(
            genome_version=genome_version).process_data(variant_data)

        # print("Response ",variant_data)
        self.assertEqual(variant_data["chr7:140753336A>T"]["molecular_features"]["aromaticity_alt"], 0, "")

        op.write_file(outfile, variant_data, file_type="vcf")

        file = open(outfile)
        contents = file.read()[0:700]
        print(contents)
        contents_expected = ('##reference=hg38\n'
 '#CHROM\tPOS\tID\tREF\tALT\tQUAL\tFILTER\tINFO\n'
 'chr7\t140753336\t.\tA\tT\t.\t.\t'
 'CHROM=7;POS_hg38=140753336;REF=A;ALT=T;gene_name=BRAF;variant_exchange=V600E;transcript=NM_004333.6;variant_exchange_long=Val600Glu;molecular_weight_alt=147.1293;molecular_weight_ref=117.1463;molecular_weight_diff=29.98;charge_at_pH_7_4_ref=-0.47684917889778056;charge_at_pH_7_4_alt=-1.3305060940156797;charge_at_pH_7_4_diff=-0.85;aromaticity_ref=0;aromaticity_alt=0;aromaticity_diff=0.0;flexibility_ref_str=low;flexibility_alt_str=high;flexibility_ref=0;flexibility_alt=2;flexibility_diff=2.0;ionizable_ref=0;ionizable_alt=1;ionizable_diff=1.0;polarity_ref_str=nonpolar;polarity_alt_str=polar;polarity_ref=0;polarity_a')
        self.assertEqual(contents_expected,contents,"")
        file.close()

    def test_molfeat_special_vars(self):
        genome_version = 'hg38'
        __location__ = os.path.realpath(
            os.path.join(os.getcwd(), os.path.dirname(__file__)))
        infile = __location__ + "/../../test_files/cl_sample.vcf"
        outfile = __location__ + "/../../test_files/cl_sample.molfeat.vcf"

        bframe = op.read_file(infile)
        print(bframe.data)
        self.assertEqual(len(bframe.data.keys()),22, "")

        #variant_data = op.UTAAdapterClient(genome_version=genome_version).process_data(data)
        magic_obj = op.MolecularFeaturesClient(
            genome_version=genome_version)
        #op.write_file(outfile, variant_data)

        #ag.process_file(infile, outfile, magic_obj=magic_obj)

        bframe.data = op.MolecularFeaturesClient(genome_version=genome_version).process_data(bframe.data)

        #file = open(outfile)
        #contents = file.read()[0:60]
        #contents_expected = """n"""
        #self.assertEqual(contents, contents_expected, "")
        #file.close()
        self.assertEqual(bframe.data["chr19:13135702G>A"]["molecular_features"]["h_bond_acceptor_alt"],0,"")




import os, configparser

# read in config.ini
config = configparser.ConfigParser()
config.read(os.path.join(os.path.dirname(__file__), '', 'config.ini'))

def get_config(client_config=None):
    if client_config is None:
        return config
    else:
        # Merge client configuration and default configuration
        return config | client_config

if "ONKOPUS_MODULE_SERVER" in os.environ:
    __MODULE_SERVER__ = os.getenv("ONKOPUS_MODULE_SERVER")
    print("Onkopus module server: ",__MODULE_SERVER__)
else:
    __MODULE_SERVER__ = config['DEFAULT']['ONKOPUS_MODULE_SERVER']

if "ONKOPUS_MODULE_SERVER2" in os.environ:
    __MODULE_SERVER2__ = os.getenv("ONKOPUS_MODULE_SERVER2")
else:
    __MODULE_SERVER2__ = "134.76.19.66" #__MODULE_SERVER__

if "ONKOPUS_MODULE_PROTOCOL" in os.environ:
    __MODULE_PROTOCOL__ = os.getenv("ONKOPUS_MODULE_PROTOCOL")
    print("Onkopus module protocol: ",__MODULE_PROTOCOL__)
else:
    __MODULE_PROTOCOL__ = config['DEFAULT']['ONKOPUS_MODULE_PROTOCOL']

if "ONKOPUS_MODULE_PROTOCOL2" in os.environ:
    __MODULE_PROTOCOL2__ = os.getenv("ONKOPUS_MODULE_PROTOCOL2")
    #print("Onkopus module protocol: ",__MODULE_PROTOCOL__)
else:
    __MODULE_PROTOCOL2__ = "http" #__MODULE_PROTOCOL__

#if "MODULE_PROTOCOL" in os.environ:
#    __MODULE_PROTOCOL__ = os.getenv("MODULE_PROTOCOL")
#else:
#    __MODULE_PROTOCOL__ = config['DEFAULT']['MODULE_PROTOCOL']

if "ONKOPUS_PORTS_ACTIVE" in os.environ:
    __PORTS_ACTIVE__ = os.getenv("ONKOPUS_PORTS_ACTIVE")
    print("Onkopus module ports active: ",__PORTS_ACTIVE__)
else:
    __PORTS_ACTIVE__ = config['DEFAULT']['ONKOPUS_PORTS_ACTIVE']

if "ONKOPUS_PROXY_PORT" in os.environ:
    __PROXY_PORT__ = os.getenv("ONKOPUS_PROXY_PORT")
else:
    __PROXY_PORT__ = config['DEFAULT']['ONKOPUS_PROXY_PORT']

if "ONKOPUS_SYS_DIR" in os.environ:
    __DATA_DIR_SYS__ = os.getenv('ONKOPUS_SYS_DIR')
else:
    __DATA_DIR_SYS__ = config['DEFAULT']['ONKOPUS_SYS_DIR']

if "ONKOPUS_DATA_DIR" in os.environ:
    __DATA_DIR__ = os.getenv('ONKOPUS_DATA_DIR')
else:
    __DATA_DIR__ = config['DEFAULT']['ONKOPUS_DATA_DIR']

if "ONKOPUS_DATA_DIR_PUB" in os.environ:
    __DATA_DIR_PUB__ = os.getenv('ONKOPUS_DATA_DIR_PUB')
else:
    __DATA_DIR_PUB__ = config['DEFAULT']['ONKOPUS_DATA_DIR_PUB']

if "REDIS_SERVER" in os.environ:
    __REDIS_SERVER__ = os.getenv("REDIS_SERVER")
else:
    __REDIS_SERVER__ = config['ONKOPUSDB']['REDIS_SERVER']

if "REDIS_SERVER_PORT" in os.environ:
    __REDIS_SERVER_PORT__ = os.getenv("REDIS_SERVER_PORT")
else:
    __REDIS_SERVER_PORT__ = config['ONKOPUSDB']['REDIS_SERVER_PORT']

if "REDIS_SERVER_DB" in os.environ:
    __REDIS_SERVER_DB__ = os.getenv("REDIS_SERVER_DB")
else:
    __REDIS_SERVER_DB__ = config['ONKOPUSDB']['REDIS_SERVER_DB']

__ID_FILE__= __DATA_DIR__ + "/ids.txt"
__ID_FILE_PUB__= __DATA_DIR_PUB__ + "/ids.txt"
__FEATURE_FILE__= "features.csv"
__META_FILE__="meta.csv"
__UPLOAD_FILE__="files.csv"
__HIST_FILE__="history.csv"

__ACTIVE_MODULES__ = config['DEFAULT']['ACTIVE_MODULES'].split(",")

__FEATURE_GENE__ = 'gene_name'
__FEATURE_VARIANT__ = 'variant_exchange'
__FEATURE_QID__ = 'q_id'

__VCF_FILE__ = 'variants.vcf.gz'
__VCF_FILTERED_FILE__ = 'filtered.vcf.gz'
__VCF_ANNOTATED_FILE__ = 'annotated.vcf.gz'

if "LIFTOVER_DATA_DIR" in os.environ:
    __LIFTOVER_DATA_DIR__ = os.getenv('LIFTOVER_DATA_DIR')
else:
    #__LIFTOVER_DATA_DIR__ = config['DEFAULT']['LIFTOVER_DATA_DIR']
    __location__ = os.path.realpath(
        os.path.join(os.getcwd(), os.path.dirname(__file__)))
    __LIFTOVER_DATA_DIR__ = __location__ + '/data'

__LIFTOVER_FILE_HG38 = "hg38ToHg19.over.chain.gz"
__LIFTOVER_FILE_HG19 = "hg19ToHg38.over.chain.gz"


__location__ = os.path.realpath(
        os.path.join(os.getcwd(), os.path.dirname(__file__)))
__LOGFILE__ = __location__ + '/error.log'

#if "MODULE_SERVER" in os.environ:
#    __MODULE_SERVER__ = os.getenv("MODULE_SERVER")
#else:
#    __MODULE_SERVER__ = config['DEFAULT']['MODULE_SERVER']

#if "PORTS_ACTIVE" in os.environ:
#    __PORTS_ACTIVE__ = os.getenv("PORTS_ACTIVE")
#else:
#    __PORTS_ACTIVE__ = config['DEFAULT']['PORTS_ACTIVE']

if "SYS_DIR" in os.environ:
    __DATA_DIR_SYS__ = os.getenv('SYS_DIR')
else:
    __DATA_DIR_SYS__ = config['DEFAULT']['SYS_DIR']

# MODULE PORTS
__PORT_DBSNP__ = ':8090'
__PORT_CLINVAR__ = ':8092'
__PORT_METAKB__ = ':8100'
__PORT_UTAADAPTER__ = ':8084'
__PORT_REVEL__ = ':8096'
__PORT_LOFTOOL__ = ':8094'
__PORT_VUSPREDICT__ = ':9000'
__PORT_MVP__ = ':10108'
__PORT_CIVIC__ = ':10106'
__PORT_ONCOKB__ = ':10112'
__PORT_ONKOPUS_AGGREGATOR__ = ':10110'
__PORT_ONKOPUS_INTERPRETER__ = ':10114'
__PORT_ONKOPUS_PLOTS__ = ':10116'
__PORT_PRIMATEAI__ = ':10120'
__PORT_DBNSFP__ = ':10122'
__PORT_DRUGCLASS__ = ':11010'
__PORT_GENCODE__ = ':10132'
__PORT_DGIDB_DB__ = ':10148'
__PORT_DGIDB_ADAPTER__ = ':10150'
__PORT_ALPHAMISSENSE__ = ':10162'
__PORT_PROTEIN_MODULE__ = ':10400'
__PORT_ONKOPUS_WEB__ = ':10102'
__PORT_COSMIC__ = ':10136'
__PORT_SCANNET__ = ':10166'
__PORT_ONKOPUS_SERVER__ = ':10100'
__PORT_MOLECULAR_FEATURES__ = ':10174'
__PORT_GE_MODULE__ = ':10180'
__PORT_DS__ = ':10190'
__PORT_CNASPHERE__ = ':10980'
__PORT_CNVOYANT__ = ':10184'
__PORT_CLASSIFYCNV__ = ':10188'
__PORT_XCNV__ = ':10186'
__PORT_ISV__ = ':10194'
__PORT_TADA__ = ':10196'
__PORT_DBCNV__ = ':10200'
__PORT_STRVCTVRE__ = ':10198'
__PORT_SVFX__ = ':10202'

if __PORTS_ACTIVE__ != "1":
    __PORT_DBSNP__ = ''
    __PORT_CLINVAR__ = ''
    __PORT_METAKB__ = ''
    __PORT_UTAADAPTER__ = ''
    __PORT_REVEL__ = ''
    __PORT_LOFTOOL__ = ''
    __PORT_VUSPREDICT__ = ''
    __PORT_MVP__ = ''
    __PORT_CIVIC__ = ''
    __PORT_ONCOKB__ = ''
    __PORT_ONKOPUS_AGGREGATOR__ = ''
    __PORT_ONKOPUS_INTERPRETER__ = ''
    __PORT_PRIMATEAI__ = ''
    __PORT_DBNSFP__ = ''
    __PORT_DRUGCLASS__ = ''
    __PORT_GENCODE__ = ''
    __PORT_ALPHAMISSENSE__ = ''
    __PORT_PROTEIN_MODULE__ = ''
    __PORT_ONKOPUS_WEB__ = ''
    __PORT_ONKOPUS_PLOTS__ = ''
    __PORT_COSMIC__ = ''
    __PORT_SCANNET__ = ''
    __PORT_ONKOPUS_SERVER__ = ''
    __PORT_MOLECULAR_FEATURES__ = ''
    __PORT_GE_MODULE__ = ''
    __PORT_DGIDB_ADAPTER__ = ''
    __PORT_DS__ = ''
    __PORT_CNASPHERE__ = ''
    __PORT_CNVOYANT__ = ''
    __PORT_CLASSIFYCNV__ = ''
    __PORT_XCNV__ = ''
    __PORT_ISV__ = ''
    __PORT_TADA__ = ''
    __PORT_DBCNV__ = ''
    __PORT_STRVCTVRE__ = ''
    __PORT_SVFX__ = ''
elif __PROXY_PORT__ != "":
    __PORT_DBSNP__ = __PROXY_PORT__
    __PORT_CLINVAR__ = __PROXY_PORT__
    __PORT_METAKB__ = __PROXY_PORT__
    __PORT_UTAADAPTER__ = __PROXY_PORT__
    __PORT_REVEL__ = __PROXY_PORT__
    __PORT_LOFTOOL__ = __PROXY_PORT__
    __PORT_VUSPREDICT__ = __PROXY_PORT__
    __PORT_MVP__ = __PROXY_PORT__
    __PORT_CIVIC__ = __PROXY_PORT__
    __PORT_ONCOKB__ = __PROXY_PORT__
    __PORT_ONKOPUS_AGGREGATOR__ = __PROXY_PORT__
    __PORT_ONKOPUS_INTERPRETER__ = __PROXY_PORT__
    __PORT_PRIMATEAI__ = __PROXY_PORT__
    __PORT_DBNSFP__ = __PROXY_PORT__
    __PORT_DRUGCLASS__ = __PROXY_PORT__
    __PORT_GENCODE__ = __PROXY_PORT__
    __PORT_ALPHAMISSENSE__ = __PROXY_PORT__
    __PORT_PROTEIN_MODULE__ = __PROXY_PORT__
    __PORT_ONKOPUS_WEB__ = __PROXY_PORT__
    __PORT_ONKOPUS_PLOTS__ = __PROXY_PORT__
    __PORT_COSMIC__ = __PROXY_PORT__
    __PORT_SCANNET__ = __PROXY_PORT__
    __PORT_ONKOPUS_SERVER__ = __PROXY_PORT__
    __PORT_MOLECULAR_FEATURES__ = __PROXY_PORT__
    __PORT_GE_MODULE__ = __PROXY_PORT__
    __PORT_DGIDB_ADAPTER__ = __PROXY_PORT__
    __PORT_DS__ = __PROXY_PORT__
    __PORT_CNASPHERE__ = __PROXY_PORT__
    __PORT_CNVOYANT__ = __PROXY_PORT__
    __PORT_CLASSIFYCNV__ = __PROXY_PORT__
    __PORT_XCNV__ = __PROXY_PORT__
    __PORT_ISV__ = __PROXY_PORT__
    __PORT_TADA__ = __PROXY_PORT__
    __PORT_DBCNV__ = __PROXY_PORT__
    __PORT_STRVCTVRE__ = __PROXY_PORT__
    __PORT_SVFX__ = __PROXY_PORT__

__FEATURE_GENE__ = 'gene_name'
__FEATURE_VARIANT__ = 'variant_exchange'
__FEATURE_QID__ = 'q_id'

error_logfile = __DATA_DIR_SYS__ + 'error'
concat_char = "_"
vcf_header_key = 'vcf_header'
variant_data_key = 'variant_data'

# Molecular features
molecular_features_info_lines= [
        '##INFO=<ID=molecular_features_molecular_weight_alt,Number=1,Type=Float,Description="Molecular weight of alternate amino acid">',
'##INFO=<ID=molecular_features_molecular_weight_ref,Number=1,Type=Float,Description="Molecular weight of reference amino acid">',
'##INFO=<ID=molecular_features_molecular_weight_diff,Number=1,Type=Float,Description="Molecular weight difference between reference and alternate amino acid">',
'##INFO=<ID=molecular_features_charge_at_pH_7_4_ref,Number=1,Type=Float,Description="Charge of reference amino acid">',
'##INFO=<ID=molecular_features_charge_at_pH_7_4_alt,Number=1,Type=Float,Description="Charge of alternate amino acid">',
'##INFO=<ID=molecular_features_charge_at_pH_7_4_diff,Number=1,Type=Float,Description="Charge difference between reference and alternate amino acid">',
'##INFO=<ID=molecular_features_aromaticity_alt,Number=1,Type=Integer,Description="Aromaticity of alternate amino acid">',
'##INFO=<ID=molecular_features_aromaticity_ref,Number=1,Type=Integer,Description="Aromaticity of reference amino acid">',
'##INFO=<ID=molecular_features_aromaticity_diff,Number=1,Type=Float,Description="Aromaticity difference between reference and alternate amino acid">',
'##INFO=<ID=molecular_features_flexibility_alt_str,Number=1,Type=String,Description="Flexibility of alternate amino acid">',
'##INFO=<ID=molecular_features_flexibility_ref_str,Number=1,Type=String,Description="Flexibility of reference amino acid">',
'##INFO=<ID=molecular_features_flexibility_ref,Number=1,Type=Integer,Description="Flexibility of reference amino acid (numeric)">',
'##INFO=<ID=molecular_features_flexibility_alt,Number=1,Type=Integer,Description="Flexibility of alternate amino acid (numeric)">',
'##INFO=<ID=molecular_features_flexibility_diff,Number=1,Type=Float,Description="Difference in flexibility between reference and alternate amino acid">',
'##INFO=<ID=molecular_features_ionizable_alt,Number=1,Type=Integer,Description="Ionization potential of alternate amino acid">',
'##INFO=<ID=molecular_features_ionizable_ref,Number=1,Type=Integer,Description="Ionization potential of reference amino acid">',
'##INFO=<ID=molecular_features_ionizable_diff,Number=1,Type=Float,Description="Ionization potential difference between reference and alternate amino acid">',
'##INFO=<ID=molecular_features_polarity_ref_str,Number=1,Type=String,Description="Polarity of reference amino acid">',
'##INFO=<ID=molecular_features_polarity_alt_str,Number=1,Type=Integer,Description="Polarity of alternate amino acid">',
'##INFO=<ID=molecular_features_polarity_ref,Number=1,Type=Integer,Description="Polarity of reference amino acid (numeric)">',
'##INFO=<ID=molecular_features_polarity_diff,Number=1,Type=Integer,Description="Polarity of alternate amino acid (numeric)">',
'##INFO=<ID=molecular_features_solubility_ref_str,Number=1,Type=String,Description="Solubility of reference amino acid">',
'##INFO=<ID=molecular_features_solubility_alt_str,Number=1,Type=String,Description="Solubility of alternate amino acid">',
'##INFO=<ID=molecular_features_solubility_ref,Number=1,Type=Integer,Description="Solubility of reference amino acid">',
'##INFO=<ID=molecular_features_solubility_alt,Number=1,Type=Integer,Description="Solubility of alternate amino acid">',
'##INFO=<ID=molecular_features_solubility_diff,Number=1,Type=Float,Description="Solubility difference between reference and alternate amino acid">',
'##INFO=<ID=molecular_features_h_bond_acceptor_ref,Number=1,Type=Integer,Description="Number of hydrogen bond acceptors of reference amino acid">',
'##INFO=<ID=molecular_features_h_bond_acceptor_alt,Number=1,Type=Integer,Description="Number of hydrogen bond acceptors of alternate amino acid">',
'##INFO=<ID=molecular_features_h_bond_acceptor_diff,Number=1,Type=Float,Description="Difference of hydrogen bond acceptors between reference and alternate amino acid">',
'##INFO=<ID=molecular_features_h_bond_donor_alt,Number=1,Type=Integer,Description="Number of hydrogen bond donors of alternate amino acid">',
'##INFO=<ID=molecular_features_h_bond_donor_ref,Number=1,Type=Integer,Description="MNumber of hydrogen bond donors of reference amino acid">',
'##INFO=<ID=molecular_features_h_bond_donor_diff,Number=1,Type=Float,Description="Differenceof hydrogen bond donors between reference and alternate amino acid">',
'##INFO=<ID=molecular_features_phosphorylation_ref,Number=1,Type=Integer,Description="Phosphorylation potential of reference amino acid">',
'##INFO=<ID=molecular_features_phosphorylation_alt,Number=1,Type=Integer,Description="Phosphorylation potential of alternate amino acid">',
'##INFO=<ID=molecular_features_phosphorylation_diff,Number=1,Type=Float,Description="Difference between phosphorylation potential between reference and alternate amino acid">',
'##INFO=<ID=molecular_features_hydropathy_ref_str,Number=1,Type=String,Description="Hydropathy of reference amino acid">',
'##INFO=<ID=molecular_features_hydropathy_alt_str,Number=1,Type=String,Description="Hydropathy of alternate amino acid">',
'##INFO=<ID=molecular_features_hydropathy_ref,Number=1,Type=Integer,Description="Hydropathy of reference amino acid (numeric)">',
'##INFO=<ID=molecular_features_hydropathy_alt,Number=1,Type=Integer,Description="Hydropathy of alternate amino acid (numeric)">',
'##INFO=<ID=molecular_features_hydropathy_diff,Number=1,Type=Float,Description="Difference of hydropathy between reference and alternate amino acid">',
'##INFO=<ID=molecular_features_alpha_helix_breaker_alt,Number=1,Type=String,Description="Alpha helix breaker potential of alternate amino acid">',
'##INFO=<ID=molecular_features_alpha_helix_breaker_ref,Number=1,Type=String,Description="Alpha helix breaker potential of reference amino acid">',
'##INFO=<ID=molecular_features_alpha_helix_breaker_diff,Number=1,Type=String,Description="Difference in alpha helix breaker potential between reference and alternate amino acid">',
'##INFO=<ID=molecular_features_beta_sheet_propensity_alt,Number=1,Type=String,Description="Beta sheet propensity of alternate amino acid">',
'##INFO=<ID=molecular_features_beta_sheet_propensity_ref,Number=1,Type=String,Description="Beta sheet propensity of reference amino acid">',
'##INFO=<ID=molecular_features_beta_sheet_propensity_diff,Number=1,Type=String,Description="Difference in beta sheet propensity between reference and alternate amino acid">',
'##INFO=<ID=molecular_thermal_stability_alt,Number=1,Type=String,Description="Thermal stability of alternate amino acid">',
'##INFO=<ID=molecular_features_thermal_stability_ref,Number=1,Type=String,Description="Thermal stability of reference amino acid">',
'##INFO=<ID=molecular_features_thermal_stability_alt,Number=1,Type=String,Description="Thermal stability of alternate amino acid">',
'##INFO=<ID=molecular_features_thermal_stability_diff,Number=1,Type=String,Description="Difference in thermal stability between reference and alternate amino acid">',
    ]
molecular_features_src = __MODULE_PROTOCOL__ + "://" + __MODULE_SERVER__ + __PORT_MOLECULAR_FEATURES__ + "/molecular-features/v1/MolecularFeatures"
molecular_features_srv_prefix= "molecular_features"
molecular_features_keys = [ 'molecular_weight_alt','molecular_weight_ref','molecular_weight_diff',
                            'charge_at_pH_7_4_ref','charge_at_pH_7_4_alt','charge_at_pH_7_4_diff',
                            'aromaticity_ref_str', 'aromaticity_alt_str','aromaticity_ref', 'aromaticity_alt','aromaticity_diff',
                            'flexibility_ref_str','flexibility_alt_str','flexibility_ref','flexibility_alt','flexibility_diff',
                            'ionizable_ref_str','ionizable_alt_str','ionizable_ref','ionizable_alt','ionizable_diff',
                            'polarity_ref_str', 'polarity_alt_str','polarity_ref', 'polarity_alt','polarity_diff',
                            'solubility_ref_str', 'solubility_alt_str','solubility_ref', 'solubility_alt','solubility_diff',
                            'h_bond_acceptor_ref_str', 'h_bond_acceptor_alt_str','h_bond_acceptor_ref', 'h_bond_acceptor_alt','h_bond_acceptor_diff',
                            'h_bond_donor_ref_str', 'h_bond_donor_alt_str','h_bond_donor_ref', 'h_bond_donor_alt','h_bond_donor_diff',
                            'phosphorylation_ref_str', 'phosphorylation_alt_str','phosphorylation_ref', 'phosphorylation_alt','phosphorylation_diff',
                            'hydropathy_ref_str', 'hydropathy_alt_str','hydropathy_ref', 'hydropathy_alt','hydropathy_diff',
                            'alpha_helix_breaker_alt', 'alpha_helix_breaker_ref', 'alpha_helix_breaker_diff',
                            'beta_sheet_propensity_alt', 'beta_sheet_propensity_ref', 'beta_sheet_propensity_diff',
                            'thermal_stability_alt', 'thermal_stability_ref', 'thermal_stability_diff'
                            ]
molecular_features_response_keys= []

# Onkopus Interpreter
onkopus_interpreter_info_lines= [
        '##INFO=<ID=Onkopus_Interpreter_Score,Number=1,Type=String,Description="Variant classification score">',
    ]
onkopus_interpreter_src = __MODULE_PROTOCOL__ + "://" + __MODULE_SERVER__ + __PORT_ONKOPUS_INTERPRETER__ + "/onkopus-interpreter/v1/compute_pathogenicity_prediction"
onkopus_interpreter_srv_prefix= "Onkopus_Interpreter"
onkopus_interpreter_keys = [ 'position' ]
onkopus_interpreter_response_keys= []

# Onkopus Aggregator
onkopus_aggregator_info_lines=[
        '##INFO=<ID=Onkopus_Aggregator,Number=1,Type=String,Description="">',
    ]
onkopus_aggregator_src = __MODULE_PROTOCOL__ + "://" + __MODULE_SERVER__ + __PORT_ONKOPUS_AGGREGATOR__ + "/onkopus-aggregator/v1/aggregate_treatment_data"
onkopus_aggregator_srv_prefix= "onkopus_aggregator"
onkopus_aggregator_keys = [ 'clinsig_summary' ]
onkopus_aggregator_response_keys= []

# UTA Adapter
uta_adapter_src = __MODULE_PROTOCOL__ + "://" + __MODULE_SERVER__ + __PORT_UTAADAPTER__ + "/CCS/v1/GenomicToGene/{}/"
uta_genomic_keys = ['gene_name', 'variant_exchange', 'input_data', 'transcript', 'variant', 'variant_exchange_long']
uta_genomic_key_labels = []
uta_gene_keys = ['results_string']
uta_gene_response_keys = ['chr', 'start', 'ref', 'var', 'input_data']
uta_adapter_genetogenomic_src = __MODULE_PROTOCOL__ + "://" + __MODULE_SERVER__ + __PORT_UTAADAPTER__ + "/CCS/v1/GeneToGenomic/{}/"
uta_adapter_info_lines = [
        '##INFO=<ID=UTA_Adapter_gene_name,Number=1,Type=String,Description="Gene name of a variant">',
        '##INFO=<ID=UTA_Adapter_variant_exchange,Number=1,Type=String,Description="Variant exchange of a genomic location">',
        '##INFO=<ID=UTA_Adapter_input_data,Number=1,Type=String,Description="SeqCAT notation of a variant">',
        '##INFO=<ID=UTA_Adapter_transcript,Number=1,Type=String,Description="MANE Select transcript of a variant">',
        '##INFO=<ID=UTA_Adapter_variant,Number=1,Type=String,Description="RefSeq protein notation of a variant">',
        '##INFO=<ID=UTA_Adapter_variant_exchange_long,Number=1,Type=String,Description="Variant amino acid exchange in 3-letter notation">'
    ]
uta_adapter_srv_prefix = 'UTA_Adapter'
uta_adapter_genetogenomic_srv_prefix = 'UTA_Adapter_gene'
uta_adapter_genetogenomic_gene_prefix = 'Gene'
uta_adapter_genetogenomic_variant_prefix = 'Variant'
uta_adapter_genetogenomic_extract_keys = [ 'results_string', 'c_dna_string', 'refCodon', 'varCodon', 'nucleotide_change',
                                           'strand', 'refAmino', 'varAmino', 'cds_start', 'cds_end' ]
uta_adapter_genetogenomic_info_lines = []

uta_adapter_transcripttogenomic_src = __MODULE_PROTOCOL__ + "://" + __MODULE_SERVER__ + __PORT_UTAADAPTER__ + "/CCS/v1/cdnaToGene/{}/"
uta_adapter_transcripttogenomic_srv_prefix = 'UTA_Adapter_transcript'
uta_adapter_transcripttogenomic_extract_keys = []

uta_adapter_indeltogene_srv_prefix = 'UTA_Adapter_indel'
uta_adapter_indeltogene_extract_keys = []
uta_adapter_indeltogene_src = __MODULE_PROTOCOL__ + "://" + __MODULE_SERVER__ + __PORT_UTAADAPTER__ + "/CCS/v1/IndelToGene/{}/"

uta_adapter_liftover_srv_prefix = "UTA_Adapter_liftover"
uta_adapter_liftover_src = __MODULE_PROTOCOL__ + "://" + __MODULE_SERVER__ + __PORT_UTAADAPTER__ + "/CCS/v1/liftover/{}/"
uta_adapter_liftover_info_lines = [
        '##INFO=<ID=UTA-Adapter-LiftOver,Number=1,Type=String,Description="Reference Genome LiftOver">'
    ]
uta_liftover_gene_keys = [ 'position' ]
uta_liftover_response_keys = ['position']

uta_adapter_genefusion_src = __MODULE_PROTOCOL__ + "://" + __MODULE_SERVER__ + __PORT_UTAADAPTER__ + "/CCS/v1/fusionIsInFrame/{}/"
uta_adapter_genefusion_srv_prefix = 'UTA_Adapter_fusion'

# UTA-Adapter Protein Sequence
uta_adapter_protein_sequence_info_lines = [
        '##INFO=<ID=UTA_Adapter_protein_sequence_protein_sequence,Number=1,Type=String,Description="Protein sequence (MANE Select)">'
    ]
uta_adapter_protein_sequence_src = __MODULE_PROTOCOL__ + "://" + __MODULE_SERVER__ + __PORT_UTAADAPTER__ + "/CCS/v1/get_protein_sequence/"
uta_adapter_protein_sequence_srv_prefix = "UTA_Adapter_protein_sequence"
uta_adapter_protein_sequence_keys = ['protein_sequence']

# dbSNP
dbsnp_src = __MODULE_PROTOCOL__ + "://" + __MODULE_SERVER__ + __PORT_DBSNP__ + "/dbsnp/v1/{}/full?genompos="
dbsnp_srv_prefix = 'dbsnp'
dbsnp_keys = ['rsID', 'freq_total','freq_african','freq_africanOthers','freq_asian','freq_eastAsian','freq_european','freq_latinAmerican1','freq_latinAmerican2',
              'freq_other', 'freq_otherAsian', 'freq_southAsian']
dbsnp_return_keys = ['dbsnpRSID','']
dbsnp_srv_prefix = 'dbsnp'
dbsnp_info_lines = [
    '##INFO=<ID=dbsnp_rsID,Number=1,Type=String,Description="dbSNP reference ID  (rs or RefSNP)">',
    '##INFO=<ID=dbsnp_freq_total,Number=1,Type=Float,Description="Population allele frequency (Total)">',
    '##INFO=<ID=dbsnp_freq_african,Number=1,Type=Float,Description="Population allele frequency (African)">',
    '##INFO=<ID=dbsnp_freq_africanOthers,Number=1,Type=Float,Description="Population allele frequency (African (others))">',
    '##INFO=<ID=dbsnp_freq_asian,Number=1,Type=Float,Description="Population allele frequency (Asian)">',
    '##INFO=<ID=dbsnp_freq_eastAsian,Number=1,Type=Float,Description="Population allele frequency (East Asian)">',
    '##INFO=<ID=dbsnp_freq_european,Number=1,Type=Float,Description="Population allele frequency (European)">',
    '##INFO=<ID=dbsnp_freq_latinAmerican1,Number=1,Type=Float,Description="Population allele frequency (Latin American(1))">',
    '##INFO=<ID=dbsnp_freq_latinAmerican2,Number=1,Type=Float,Description="Population allele frequency (Latin American(2))">',
    '##INFO=<ID=dbsnp_freq_other,Number=1,Type=Float,Description="Population allele frequency (Other)">',
    '##INFO=<ID=dbsnp_freq_otherAsian,Number=1,Type=Float,Description="Population allele frequency (Other(Asian))">',
    '##INFO=<ID=dbsnp_freq_southAsian,Number=1,Type=Float,Description="Population allele frequency (South Asian)">'
]

# ClinVar
clinvar_src = __MODULE_PROTOCOL__ + "://" + __MODULE_SERVER__ + __PORT_CLINVAR__ + "/clinvar/v1/{}/full?genompos="
clinvar_keys = ['CLNSIG','CLNREVSTAT','CLINVARID','CLNDN','ALLELEID']
clinvar_srv_prefix = config['MODULES']['CLINVAR_PREFIX']
clinvar_info_lines = ['##INFO=<ID=clinvar_CLNSIG,Number=1,Type=String,Description="Estimated pathogenicity of genomic alteration">',
                      '##INFO=<ID=clinvar_CLNREVSTAT,Number=1,Type=String,Description="Review status of the ClinVar clinical significance estimation">',
                      '##INFO=<ID=clinvar_CLINVARID,Number=1,Type=String,Description="Variant Clinvar ID">',
                      '##INFO=<ID=clinvar_CLNDN,Number=1,Type=String,Description="Diseases associated with the variant>"',
                      '##INFO=<ID=clinvar_ALLELEID,Number=1,Type=String,Description="Allele ID">'
                      ]

# REVEL
revel_src = __MODULE_PROTOCOL__ + "://" + __MODULE_SERVER__ + __PORT_REVEL__ + "/revel/v1/{}/full?genompos="
revel_srv_prefix = config['MODULES']['REVEL_PREFIX']
revel_keys = ['Score']
revel_response_keys = ['revelScore']
revel_info_lines = ['##INFO=<ID=REVEL_Score,Number=1,Type=String,Description="REVELScore">']

# LoFTool
loftool_src = __MODULE_PROTOCOL__ + "://" + __MODULE_SERVER__ + __PORT_LOFTOOL__ + "/loftool/v1/{}/full?"
loftool_srv_prefix = config['MODULES']['LOFTOOL_PREFIX']
loftool_keys = ['Score']
loftool_response_keys = ['loftoolScore']
loftool_info_lines = ['##INFO=<ID=LoFTool-Score,Number=1,Type=String,Description="LoFTool-Score">']

# VUS-Predict
vuspredict_src = __MODULE_PROTOCOL__ + "://" + __MODULE_SERVER__ + __PORT_VUSPREDICT__ + "/VUS/v1/{}/toPipeline?"
vuspredict_srv_prefix = config['MODULES']['VUSPREDICT_PREFIX']
vuspredict_keys = ['FATHMM', 'Missense3D', 'SIFT', 'Score', 'Pipeline']
vuspredict_response_keys = ['vusFATHMM', 'vusMissense3D', 'vusSIFT', 'vusScore', 'vusPipeline']
vuspredict_info_lines = ['##INFO=<ID=VUS-Predict-Score,Number=1,Type=String,Description="VUS-Predict-Score">']

# PrimateAI
primateai_info_lines = ['##INFO=<ID=PrimateAI-Score,Number=1,Type=String,Description="PrimateAI-Score">']
primateai_src = __MODULE_PROTOCOL__ + "://" + __MODULE_SERVER__ + __PORT_PRIMATEAI__ + "/primateai-adapter/v1/{}/full?genompos="
primateai_srv_prefix = config['MODULES']['PRIMATEAI_PREFIX']
primateai_keys = ['Score']

# AlphaMissense
alphamissense_info_lines = ['##INFO=<ID=AlphaMissense-Score,Number=1,Type=String,Description="AlphaMissense-Score">']
alphamissense_src = __MODULE_PROTOCOL__ + "://" + __MODULE_SERVER__ + __PORT_ALPHAMISSENSE__ + "/alphamissense/v1/{}/getScore?q="
alphamissense_srv_prefix = config['MODULES']['ALPHAMISSENSE_PREFIX']
alphamissense_keys = ['score']

# OncoKB
oncokb_src = __MODULE_PROTOCOL__ + "://" + __MODULE_SERVER__ + __PORT_ONCOKB__ + "/oncokb/v1/{}/full?"
oncokb_srv_prefix = config['MODULES']['ONCOKB_PREFIX']
oncokb_keys = []
oncokb_response_keys = []
oncokb_info_lines = [
]

# MetaKB
metakb_src = __MODULE_PROTOCOL__ + "://" + __MODULE_SERVER__ + __PORT_METAKB__ + "/METAKB/v1/GenomicClinicalEvidence?"
metakb_srv_prefix = config['MODULES']['METAKB_PREFIX']
metakb_keys = ['DB Variants', 'DB diseases', 'Drugs', 'Evidence Label', 'Origin Database', 'References']
metakb_response_keys = ['DBVariants', 'DBdiseases', 'Drugs', 'EvidenceLabel', 'OriginDatabase', 'References']
metakb_info_lines = [
    '##INFO=<ID=MetaKB-DB-Variants,Number=1,Type=String,Description="">',
    '##INFO=<ID=MetaKB-DB-diseases,Number=1,Type=Float,Description="">',
    '##INFO=<ID=MetaKB-Drugs,Number=1,Type=Float,Description="">',
    '##INFO=<ID=MetaKB-Evidence-Label,Number=1,Type=Float,Description="">',
    '##INFO=<ID=MetaKB-Origin-Database,Number=1,Type=Float,Description="">',
    '##INFO=<ID=MetaKB-References,Number=1,Type=Float,Description="">'
]

# MVP
mvp_src = __MODULE_PROTOCOL__ + "://" + __MODULE_SERVER__ + __PORT_MVP__ + "/mvp-adapter/v1/{}/full?genompos="
mvp_srv_prefix = config['MODULES']['MVP_PREFIX']
mvp_keys = ['Score']
mvp_response_keys = ['mvpScore']
mvp_info_lines = ['##INFO=<ID=MVP-Score,Number=1,Type=String,Description="MVP-Score">']

# dbNSFP
dbnsfp_info_lines = [
    '##INFO=<ID=dbnsfp_aaref,Number=1,Type=String,Description="Reference amino acid">',
    '##INFO=<ID=dbnsfp_aapos,Number=1,Type=Float,Description="Amino acid position within the protein sequence">',
    '##INFO=<ID=dbnsfp_codonpos,Number=1,Type=Float,Description="Variant position within the codon">',
    '##INFO=<ID=dbnsfp_refcodon,Number=1,Type=Float,Description="Reference codon">',
    '##INFO=<ID=dbnsfp_rs_dbSNP,Number=1,Type=Float,Description="dbSNP rsID">',
    '##INFO=<ID=dbnsfp_Interpro_domain,Number=1,Type=Float,Description="Protein domain">',
    '##INFO=<ID=dbnsfp_REVEL_score,Number=1,Type=Float,Description="REVEL score">',
    '##INFO=<ID=dbnsfp_AlphaMissense_score,Number=1,Type=Float,Description="AlphaMissense score">',
    '##INFO=<ID=dbnsfp_ESM1b_score,Number=1,Type=Float,Description="ESM1b score">',
    '##INFO=<ID=dbnsfp_ALFA_Total_AF,Number=1,Type=Float,Description="ALFA allele frequency (Total)">',
    '##INFO=<ID=dbnsfp_ALFA_Total_AN,Number=1,Type=Float,Description="ALFA allele frequency (Total)">',
    '##INFO=<ID=dbnsfp_EVE_score,Number=1,Type=Float,Description="EVE score">',
    '##INFO=<ID=dbnsfp_Eigen-PC-phred_coding,Number=1,Type=Float,Description="Eigen-PC phred coding score">',
    '##INFO=<ID=dbnsfp_Ensembl_geneid,Number=1,Type=Float,Description="Ensembl gene ID">',
    '##INFO=<ID=dbnsfp_Ensembl_transcriptid,Number=1,Type=Float,Description="Ensembl transcript ID (MANE Select)">',
    '##INFO=<ID=dbnsfp_Ensembl_proteinid,Number=1,Type=Float,Description="Ensembl protein ID (MANE Select)">',
    '##INFO=<ID=dbnsfp_ExAC_AC,Number=1,Type=Float,Description="ExAC allele frequency">',
    '##INFO=<ID=dbnsfp_FATHMM_score,Number=1,Type=Float,Description="FATHMM score">',
    '##INFO=<ID=dbnsfp_GENCODE_basic,Number=1,Type=Float,Description="GENCODE basic">',
    '##INFO=<ID=dbnsfp_GERP++_NR,Number=1,Type=Float,Description="GERP++ NR score">',
    '##INFO=<ID=dbnsfp_GERP++_RS,Number=1,Type=Float,Description="GERP++ RS score">',
    '##INFO=<ID=dbnsfp_LIST-S2_pred,Number=1,Type=String,Description="LIST-S2 prediction">',
    '##INFO=<ID=dbnsfp_M-CAP_pred,Number=1,Type=String,Description="M-CAP score">',
    '##INFO=<ID=dbnsfp_MPC_score,Number=1,Type=Float,Description="MPC score">',
    '##INFO=<ID=dbnsfp_MVP_score,Number=1,Type=Float,Description="MVP score">',
    '##INFO=<ID=dbnsfp_MetaLR_score,Number=1,Type=Float,Description="MetaLR score">',
    '##INFO=<ID=dbnsfp_MetaSVM_pred,Number=1,Type=String,Description="MetaSVM prediction">',
    '##INFO=<ID=dbnsfp_MutPred_score,Number=1,Type=Float,Description="MutPred score">',
    '##INFO=<ID=dbnsfp_MutationAssessor_score,Number=1,Type=Float,Description="MutationAssessor score">',
    '##INFO=<ID=dbnsfp_MutationAssessor_pred,Number=1,Type=String,Description="MutationAssessor prediction">',
    '##INFO=<ID=dbnsfp_MutationTaster_score,Number=1,Type=Float,Description="MutationTaster score">',
    '##INFO=<ID=dbnsfp_PROVEAN_pred,Number=1,Type=String,Description="PROVEAN prediction">',
    '##INFO=<ID=dbnsfp_PROVEAN_score,Number=1,Type=String,Description="PROVEAN score">',
    '##INFO=<ID=dbnsfp_PrimateAI_pred,Number=1,Type=String,Description="PrimateAI prediction">',
    '##INFO=<ID=dbnsfp_PrimateAI_score,Number=1,Type=Float,Description="PrimateAI score">',
    '##INFO=<ID=dbnsfp_SIFT_pred,Number=1,Type=String,Description="SIFT prediction">',
    '##INFO=<ID=dbnsfp_SIFT_score,Number=1,Type=Float,Description="SIFT_score">',
    '##INFO=<ID=dbnsfp_Uniprot_acc,Number=1,Type=Float,Description="Uniprot ID">',
    '##INFO=<ID=dbnsfp_Uniprot_entry,Number=1,Type=Float,Description="Uniprot entry">',
    '##INFO=<ID=dbnsfp_VARITY_ER_LOO_score,Number=1,Type=Float,Description="VARITY-ER-LOO score">',
    '##INFO=<ID=dbnsfp_VEST4_score,Number=1,Type=Float,Description="VEST4 score">',
    '##INFO=<ID=dbnsfp_gMVP_score,Number=1,Type=Float,Description="gMVP_score">',
    '##INFO=<ID=dbnsfp_gnomAD_exomes_AC,Number=1,Type=Float,Description="gnomAD exomes AC">',
    '##INFO=<ID=dbnsfp_gnomAD_exomes_AF,Number=1,Type=Float,Description="gnomAD exomes AF">',
    '##INFO=<ID=dbnsfp_AlphaMissense_rankscore,Number=1,Type=Float,Description="AlphaMissense rankscore">',
    '##INFO=<ID=dbnsfp_REVEL_rankscore,Number=1,Type=Float,Description="REVEL rankscore">',
    '##INFO=<ID=dbnsfp_PrimateAI_rankscore,Number=1,Type=Float,Description="PrimateAI rankscore">',
    '##INFO=<ID=dbnsfp_EVE_rankscore,Number=1,Type=Float,Description="EVE rankscore">',
    '##INFO=<ID=dbnsfp_SIFT4G_converted_rankscore,Number=1,Type=Float,Description="SIFT rankscore">',
    '##INFO=<ID=dbnsfp_ESM1b_rankscore,Number=1,Type=Float,Description="ESM1b rankscore">',
    '##INFO=<ID=dbnsfp_VARITY_ER_LOO_rankscore,Number=1,Type=Float,Description="VARITY ER LOO rankscore">',
    '##INFO=<ID=dbnsfp_PROVEAN_converted_rankscore,Number=1,Type=Float,Description="PROVEAN rankscore">',
    '##INFO=<ID=dbnsfp_MutationTaster_converted_rankscore,Number=1,Type=Float,Description="MutationTaster rankscore">',
    '##INFO=<ID=dbnsfp_MutationAssessor_rankscore,Number=1,Type=Float,Description="MutationAssessor rankscore">',
    '##INFO=<ID=dbnsfp_MVP_rankscore,Number=1,Type=Float,Description="MVP rankscore">',
    '##INFO=<ID=dbnsfp_fathmm-MKL_coding_rankscore,Number=1,Type=Float,Description="FATHMM-MKL coding rankscore">',
    '##INFO=<ID=dbnsfp_GERP++_RS_rankscore,Number=1,Type=Float,Description="GERP++ RS rankscore">'
]
dbnsfp_srv_prefix = config['MODULES']['DBNSFP_PREFIX']
dbnsfp_extract_keys = []
dbnsfp_src = __MODULE_PROTOCOL__ + "://" + __MODULE_SERVER__ + __PORT_DBNSFP__ + "/dbnsfp-adapter/v1/{}/full?genompos="
dbnsfp_keys = ['aaref', 'aapos', 'aapos', 'codonpos', 'refcodon', 'rs_dbSNP', 'Interpro_domain','REVEL_score',
               'AlphaMissense_score','ESM1b_score', 'ALFA_Total_AF',
               'ALFA_Total_AN', 'EVE_score', 'Eigen-PC-phred_coding','Ensembl_geneid', 'Ensembl_proteinid',
               'Ensembl_transcriptid',
               'ExAC_AC', 'Exac_AF', 'FATHMM_score', 'GENCODE_basic', 'GERP++_NR', 'GERP++_RS',
               'GTEx_V8_gene', 'GTEx_V8_tissue', 'LIST-S2_pred', 'M-CAP_pred', 'MPC_score', 'MVP_score',
               'MetaLR_score', 'MetaSVM_pred', 'MutPred_score', 'MutationAssessor_score', 'Mutation_Assessor_pred',
               'MutationTaster_score',
               'PROVEAN_pred', 'PROVEAN_score', 'PrimateAI_pred', 'PrimateAI_score', 'SIFT_pred', 'SIFT_score',
               'Uniprot_acc', 'Uniprot_entry', 'VARITY_ER_LOO_score', 'VEST4_score', 'gMVP_score',
               'gnomAD_exomes_AC', 'gnomAD_exomes_AF',
               'AlphaMissense_rankscore', 'REVEL_rankscore', 'PrimateAI_rankscore', 'EVE_rankscore',
               'SIFT4G_converted_rankscore',
               'ESM1b_rankscore', 'VARITY_ER_LOO_rankscore', 'PROVEAN_converted_rankscore',
               'MutationTaster_converted_rankscore', 'MutationAssessor_rankscore', 'MVP_rankscore',
               'fathmm-MKL_coding_rankscore', 'GERP++_RS_rankscore'
               ]

## Clinical Evidence Data

# CIViC
civic_src = __MODULE_PROTOCOL__ + "://" + __MODULE_SERVER__ + __PORT_CIVIC__ + "/civic-adapter/v1/{}/GenomicClinicalEvidence?q="
civic_gene_src = __MODULE_PROTOCOL__ + "://" + __MODULE_SERVER__ + __PORT_CIVIC__ + "/civic-adapter/v1/{}/GeneClinicalEvidence?q="
civic_srv_prefix = 'civic'
civic_keys = ['Interpretations']
civic_response_keys = ['civicData']
civic_info_lines = ['##INFO=<ID=CIViC variant interpretations,Number=1,Type=String,Description="CIViC variant interpretations">']

# DrugClass-Adapter
drugclass_src = __MODULE_PROTOCOL__ + "://" + __MODULE_SERVER__ + __PORT_DRUGCLASS__ + "/drugon2/v1/drugclass/"
drugclass_srv_prefix = 'drugclass'
drugclass_keys = ['DrugClass']
drugclass_response_keys = ['DrugClasses']
drugclass_info_lines = ['##INFO=<ID=Drug Classes,Number=1,Type=String,Description="Generated drug classes from DrugOn">']

# GENCODE-Adapter
gencode_src = __MODULE_PROTOCOL__ + "://" + __MODULE_SERVER__ + __PORT_GENCODE__ + "/gencode/v1/{}/GeneFunctionalElements"
gencode_genomic_src = __MODULE_PROTOCOL__ + "://" + __MODULE_SERVER__ + __PORT_GENCODE__ + "/gencode/v1/{}/GenomicFunctionalRegions"
gencode_mane_select_transcript_src = __MODULE_PROTOCOL__ + "://" + __MODULE_SERVER__ + __PORT_GENCODE__ + "/gencode/v1/{}/getMANESelectTranscript"
gencode_srv_prefix = 'gencode'
gencode_genomic_srv_prefix = 'gencode_genomic'
gencode_mane_select_transcript_srv_prefix = 'gencode_mane_select'
gencode_keys = ['Gencode']
gencode_info_lines = ['##INFO=<ID=Gencode,Number=1,Type=String,Description="Functional element within a gene">']

# GENCODE CNA
gencode_cna_src = __MODULE_PROTOCOL__ + "://" + __MODULE_SERVER__ + __PORT_GENCODE__ + "/gencode/v1/{}/GeneFunctionalElements"
gencode_cna_genomic_src = __MODULE_PROTOCOL__ + "://" + __MODULE_SERVER__ + __PORT_GENCODE__ + "/gencode/v1/{}/CNAFunctionalRegions"
gencode_cna_mane_select_transcript_src = __MODULE_PROTOCOL__ + "://" + __MODULE_SERVER__ + __PORT_GENCODE__ + "/gencode/v1/{}/getMANESelectTranscript"
gencode_cna_srv_prefix = 'gencode_cna'
gencode_cna_genomic_srv_prefix = 'gencode_cna'
gencode_cna_mane_select_transcript_srv_prefix = 'gencode_mane_select'
gencode_cna_keys = ['Affected_genes', 'Affected_CDS', 'Affected UTRs']
#gencode_cna_keys = ['Affected_genes']
gencode_cna_info_lines = ['##INFO=<ID=gencode_cna_Affected_genes,Number=1,Type=String,Description="Affected genes of a copy number alteration (CNA)">',
                          '##INFO=<ID=gencode_cna_Affected_CDS,Number=1,Type=String,Description="Affected coding sequences of a copy number alteration (CNA)">',
                          '##INFO=<ID=gencode_cna_Affected_UTRs,Number=1,Type=String,Description="Affected UTRs of a copy number alteration (CNA)">']

# CNV Gene Client
cnv_src = __MODULE_PROTOCOL__ + "://" + __MODULE_SERVER__ + __PORT_GENCODE__ + "/CNV/v1/annotate_cnv"
cnv_genomic_src = __MODULE_PROTOCOL__ + "://" + __MODULE_SERVER__ + __PORT_GENCODE__ + "/gencode/v1/{}/CNAFunctionalRegions"
cnv_mane_select_transcript_src = __MODULE_PROTOCOL__ + "://" + __MODULE_SERVER__ + __PORT_GENCODE__ + "/gencode/v1/{}/getMANESelectTranscript"
cnv_srv_prefix = 'cnv'
cnv_genomic_srv_prefix = 'cnv'
cnv_mane_select_transcript_srv_prefix = 'gencode_mane_select'
cnv_keys = ['Affected_genes', 'Affected_CDS', 'Affected UTRs']
#gencode_cna_keys = ['Affected_genes']
cnv_info_lines = ['##INFO=<ID=gencode_cna_Affected_genes,Number=1,Type=String,Description="Affected genes of a copy number alteration (CNA)">',
                          '##INFO=<ID=gencode_cna_Affected_CDS,Number=1,Type=String,Description="Affected coding sequences of a copy number alteration (CNA)">',
                          '##INFO=<ID=gencode_cna_Affected_UTRs,Number=1,Type=String,Description="Affected UTRs of a copy number alteration (CNA)">']

# GENCODE Genomic
gencode_srv_prefix = 'gencode'
gencode_genomic_srv_prefix = 'gencode_genomic'
gencode_mane_select_transcript_srv_prefix = 'gencode_mane_select'
gencode_genomic_keys = ['MANE_Select_transcript','transcript_list']
gencode_genomic_info_lines = ['##INFO=<ID=gencode_transcript_list,Number=1,Type=String,Description="List of Ensembl transcripts for the affected gene of a variant">',
                              ['##INFO=<ID=gencode_MANE_Select_transcript,Number=1,Type=String,Description="Ensembl identifier of the MANE Select transcript of the affected gene of a variant">']
                              ]

# DGIdb
dgidb_src = __MODULE_PROTOCOL__ + "://" + __MODULE_SERVER__ + __PORT_DGIDB_ADAPTER__ + "/dgidb/v1/GeneInteractions"
#dgidb_genomic_src = __MODULE_PROTOCOL__ + "://" + __MODULE_SERVER__ + __PORT_GENCODE__ + "/dgidb/v1/GenomicFunctionalRegions"
dgidb_srv_prefix = 'dgidb'
#dgidb_genomic_srv_prefix = 'gencode_genomic'
dgidb_keys = ['summary']
dgidb_info_lines = [
    '##INFO=<ID=dgidb_summary,Number=1,Type=String,Description="Drug-gene interactions summary">'
]

# Protein module
protein_module_src = __MODULE_PROTOCOL__ + "://" + __MODULE_SERVER__ + __PORT_PROTEIN_MODULE__ + "/protein_module/v1/plot/"
protein_module_annotations_src = __MODULE_PROTOCOL__ + "://" + __MODULE_SERVER__ + __PORT_PROTEIN_MODULE__ + "/protein_module/v1/plot_with_added_annotation_post"
protein_module_pdb_src = __MODULE_PROTOCOL__ + "://" + __MODULE_SERVER__ + __PORT_PROTEIN_MODULE__ + "/protein_module/v1/get_PDB_file"
protein_module_pdb_openfold_wt_src = __MODULE_PROTOCOL__ + "://" + __MODULE_SERVER__ + __PORT_PROTEIN_MODULE__ + "/protein_module/v1/get_PDB_file_openfold_wt"
protein_module_pdb_openfold_mutated_src = __MODULE_PROTOCOL__ + "://" + __MODULE_SERVER__ + __PORT_PROTEIN_MODULE__ + "/protein_module/v1/get_PDB_file_openfold_mutated"

# Protein domains
protein_domains_info_lines = [
    '##INFO=<ID=protein_features_RSA,Number=1,Type=String,Description="Relative accessible surface area (RSA) of variant location">'
]
protein_domains_src = ''
protein_module_domains_src = __MODULE_PROTOCOL__ + "://" + __MODULE_SERVER__ + __PORT_PROTEIN_MODULE__ + "/protein_module/v1/get_domains"
protein_domains_srv_prefix = 'protein_domains'
protein_domains_response_keys = []
protein_domains_keys = [
                         ]

# Protein features
protein_features_info_lines = [
    '##INFO=<ID=protein_features_RSA,Number=1,Type=String,Description="Relative accessible surface area (RSA) of variant location">',
'##INFO=<ID=protein_features_DSSP,Number=1,Type=Float,Description="Secondary protein structure of variant location">',
'##INFO=<ID=protein_features_Phi,Number=1,Type=Float,Description="Phi angle of variant location">',
'##INFO=<ID=protein_features_Psi,Number=1,Type=Float,Description="Psi angle of variant location">',
'##INFO=<ID=protein_features_0_NH_1_energy,Number=1,Type=Float,Description="Hydrogen bonds 0_NH_1_energy">',
'##INFO=<ID=protein_features_0_NH_1_relidx,Number=1,Type=Float,Description="Hydrogen bonds 0_NH_1_relidx">',
'##INFO=<ID=protein_features_0_NH_2_energy,Number=1,Type=Float,Description="Hydrogen bonds 0_NH_2_energy">',
'##INFO=<ID=protein_features_0_NH_2_relidx,Number=1,Type=Float,Description="Hydrogen bonds 0_NH_2_relidx">',
'##INFO=<ID=protein_features_NH_0_1_energy,Number=1,Type=Float,Description="Hydrogen bonds NH_0_1_energy">',
'##INFO=<ID=protein_features_NH_0_1_relidx,Number=1,Type=Float,Description="Hydrogen bonds NH_0_1_relidx">',
'##INFO=<ID=protein_features_NH_0_2_energy,Number=1,Type=Float,Description="Hydrogen bonds NH_0_2_energy">',
'##INFO=<ID=protein_features_NH_0_2_relidx,Number=1,Type=Float,Description="Hydrogen bonds NH_0_2_relidx">'
]
protein_features_src = ''
protein_module_features_src = __MODULE_PROTOCOL__ + "://" + __MODULE_SERVER__ + __PORT_PROTEIN_MODULE__ + "/protein_module/v1/get_features_at_mutation_point"
protein_features_srv_prefix = 'protein_features'
protein_features_response_keys = []
protein_features_keys = ['RSA','DSSP','Phi','Psi',
                         '0_NH_1_energy', '0_NH_1_relidx', '0_NH_2_energy', '0_NH_2_relidx',
                         'NH_0_1_energy', 'NH_0_1_relidx', 'NH_0_2_energy', 'NH_0_2_relidx'
                         ]

# Gene expression
ge_module_info_lines= [
'##INFO=<ID=gtex_gene_expression_per_tissue,Number=1,Type=Float,Description="GTEx gene expression per tissue">'
]
ge_module_src = __MODULE_PROTOCOL__ + "://" + __MODULE_SERVER__ + __PORT_GE_MODULE__ + "/gtex/v1/MedianExpressionPerTissue"
ge_module_srv_prefix= "gtex"
ge_module_keys = [ 'gene_expression_per_tissue' ]
ge_module_response_keys= []
# Gene expression plots
ge_plot_module_src = __MODULE_PROTOCOL__ + "://" + __MODULE_SERVER__ + __PORT_GE_MODULE__ + "/gtex/v1/MedianExpressionPerTissuePlot"



# COSMIC
cosmic_info_lines = ['##INFO=<ID=COSMIC,Number=1,Type=String,Description="COSMIC">']
cosmic_src = __MODULE_PROTOCOL__ + "://" + __MODULE_SERVER__ + __PORT_COSMIC__ + "/cosmic/v1/{}/GeneCensus"
cosmic_srv_prefix = 'cosmic'
cosmic_keys = ['cosmic']

# Gene roles
gene_role_info_lines = ['##INFO=<ID=GENE_ROLE,Number=1,Type=String,Description="Gene role according to Vogelstein et al. (2013)">']
gene_role_src = __MODULE_PROTOCOL2__ + "://" + __MODULE_SERVER2__ + __PORT_COSMIC__ + "/gene-roles/v1/getGeneRole"
gene_role_srv_prefix = 'gene_role'
gene_role_keys = ['gene_role']

# Plots
onkopus_plots_info_lines= []
onkopus_plots_src = __MODULE_PROTOCOL__ + "://" + __MODULE_SERVER__ + __PORT_ONKOPUS_PLOTS__ + "/onkopus-plots/v1/plot_data/"
onkopus_plots_srv_prefix= "onkopus-plots"
onkopus_plots_keys = [ 'position' ]
onkopus_plots_response_keys= []

# Scannet
scannet_info_lines= []
scannet_src = __MODULE_PROTOCOL__ + "://" + __MODULE_SERVER__ + __PORT_SCANNET__ + "/scannet/v1/{}/getBindingSitePrediction?protein="
scannet_srv_prefix= "scannet"
scannet_keys = [ 'binding-site-predictions' ]
scannet_response_keys= []

# CNA-Sphere Genes
cna_genes_src = __MODULE_PROTOCOL__ + "://" + __MODULE_SERVER__ + __PORT_CNASPHERE__ + "/CNV/v1/{}/get_cnv_genes"
cna_genes_genomic_src = __MODULE_PROTOCOL__ + "://" + __MODULE_SERVER__ + __PORT_CNASPHERE__ + "/CNV/v1/{}/CNAFunctionalRegions"
cna_genes_mane_select_transcript_src = __MODULE_PROTOCOL__ + "://" + __MODULE_SERVER__ + __PORT_CNASPHERE__ + "/CNV/v1/{}/getMANESelectTranscript"
cna_genes_srv_prefix = 'cna_genes'
cna_genes_genomic_srv_prefix = 'gencode_cna'
cna_genes_mane_select_transcript_srv_prefix = 'gencode_mane_select'
cna_genes_keys = ['Affected_genes', 'Affected_CDS', 'Affected UTRs']
#gencode_cna_keys = ['Affected_genes']
cna_genes_info_lines = ['##INFO=<ID=gencode_cna_Affected_genes,Number=1,Type=String,Description="Affected genes of a copy number alteration (CNA)">',
                          '##INFO=<ID=gencode_cna_Affected_CDS,Number=1,Type=String,Description="Affected coding sequences of a copy number alteration (CNA)">',
                          '##INFO=<ID=gencode_cna_Affected_UTRs,Number=1,Type=String,Description="Affected UTRs of a copy number alteration (CNA)">']

# Dosage Sensitivity
ds_src = __MODULE_PROTOCOL2__ + "://" + __MODULE_SERVER2__ + __PORT_DS__ + "/ds-adapter/v1/getGeneDosageSensitivity"
ds_srv_prefix = "dosage_sensitivity"
ds_keys = ["hi_score"]
ds_info_lines = [
    '##INFO=<ID=DS_HI_Score,Number=1,Type=String,Description="Haploinsufficiency Score">'
]

# CNVoyant
cnvoyant_src = __MODULE_PROTOCOL2__ + "://" + __MODULE_SERVER2__ + __PORT_CNVOYANT__ + "/cnvoyant-adapter/v1/{}/getScore?genompos="
cnvoyant_srv_prefix = "cnvoyant"
cnvoyant_keys = []
cnvoyant_info_lines = [
    '##INFO=<ID=DS_HI_Score,Number=1,Type=String,Description="Haploinsufficiency Score">'
]

# ClassifyCNV
classifycnv_src = __MODULE_PROTOCOL2__ + "://" + __MODULE_SERVER2__ + __PORT_CLASSIFYCNV__ + "/classifycnv-adapter/v1/{}/getScore?genompos="
classifycnv_srv_prefix = "classifycnv"
classifycnv_keys = []
classifycnv_info_lines = [
    '##INFO=<ID=DS_HI_Score,Number=1,Type=String,Description="Haploinsufficiency Score">'
]

# dbCNV
dbcnv_src = __MODULE_PROTOCOL2__ + "://" + __MODULE_SERVER2__ + __PORT_DBCNV__ + "/dbcnv-adapter/v1/{}/getScore?genompos="
dbcnv_srv_prefix = "dbcnv"
dbcnv_keys = []
dbcnv_info_lines = [
    '##INFO=<ID=DS_HI_Score,Number=1,Type=String,Description="Haploinsufficiency Score">'
]

# ISV
isv_src = __MODULE_PROTOCOL2__ + "://" + __MODULE_SERVER2__ + __PORT_ISV__ + "/isv-adapter/v1/{}/getScore?genompos="
isv_srv_prefix = "isv"
isv_keys = []
isv_info_lines = [
    '##INFO=<ID=DS_HI_Score,Number=1,Type=String,Description="Haploinsufficiency Score">'
]

# TADA
tada_src = __MODULE_PROTOCOL2__ + "://" + __MODULE_SERVER2__ + __PORT_TADA__ + "/tada-adapter/v1/{}/getScore?genompos="
tada_srv_prefix = "tada"
tada_keys = []
tada_info_lines = [
    '##INFO=<ID=DS_HI_Score,Number=1,Type=String,Description="Haploinsufficiency Score">'
]

# X-CNV
xcnv_src = __MODULE_PROTOCOL2__ + "://" + __MODULE_SERVER2__ + __PORT_XCNV__ + "/xcnv-adapter/v1/{}/getScore?genompos="
xcnv_srv_prefix = "xcnv"
xcnv_keys = []
xcnv_info_lines = [
    '##INFO=<ID=DS_HI_Score,Number=1,Type=String,Description="Haploinsufficiency Score">'
]

# Onkopus Server
onkopus_server_info_lines = []
onkopus_server_src_upload = __MODULE_PROTOCOL__ + "://" + __MODULE_SERVER__ + __PORT_ONKOPUS_SERVER__ + "/onkopus-server/v1/upload_variant_data"
onkopus_server_src_analyze_id = __MODULE_PROTOCOL__ + "://" + __MODULE_SERVER__ + __PORT_ONKOPUS_SERVER__ + "/onkopus-server/v1/analyze_variants"
onkopus_server_src_analyze_variant_request = __MODULE_PROTOCOL__ + "://" + __MODULE_SERVER__ + __PORT_ONKOPUS_SERVER__ + "/onkopus-server/v1/analyze_variant_request"
onkopus_server_srv_prefix = "onkopus-server"
onkopus_server_keys = []

# All modules client
all_modules_keys = [
    uta_genomic_keys,
    clinvar_keys,
    dbsnp_keys,
    dbnsfp_keys
]
all_modules_srv_prefix = [ "UTA_Adapter","clinvar", "dbsnp", "dbnsfp" ]

def generate_vcf_keys():
    vcf_extract_keys = []

    # UTA-Adapter
    for feature_key in uta_genomic_keys:
        vcf_extract_keys.append(uta_adapter_srv_prefix + "_" + feature_key)

    # dbSNP
    for feature_key in dbsnp_keys:
        vcf_extract_keys.append(dbsnp_srv_prefix + "_" + feature_key)

    # ClinVar
    for feature_key in clinvar_keys:
        vcf_extract_keys.append(clinvar_srv_prefix + "_" + feature_key)

    # REVEL
    for feature_key in revel_keys:
        vcf_extract_keys.append(revel_srv_prefix + "_" + feature_key)

    #LoFTool
    for feature_key in loftool_response_keys:
        vcf_extract_keys.append(loftool_srv_prefix + "_" + feature_key)

    # VUS-Predict
    for feature_key in vuspredict_response_keys:
        vcf_extract_keys.append(vuspredict_srv_prefix + "_" + feature_key)

    return vcf_extract_keys

vcf_extract_keys = generate_vcf_keys()

onkopus_modules = { dbsnp_srv_prefix: { "rsID": "", "freq_total": "" },
                    clinvar_srv_prefix: { "clinical_significance": "" },
                    uta_adapter_srv_prefix: { "gene_name": "", "variant_exchange": "" },
                    uta_adapter_genetogenomic_srv_prefix: {},
                    revel_srv_prefix: { "Score": "" },
                    loftool_srv_prefix: { "Score": ""},
                    vuspredict_srv_prefix: { "Score": ""},
                    metakb_srv_prefix: {},
                    mvp_srv_prefix: { "Score": "" },
                    primateai_srv_prefix: { "Score": ""},
                    dbnsfp_srv_prefix: { "aaref": "", "aaalt": "" },
                    civic_srv_prefix: { },
                    oncokb_srv_prefix: { "mutationEffect": { "knownEffect": "" }, "treatments": []},
                    onkopus_aggregator_srv_prefix : { "aggregated_evidence_data":{}
                        ,"meta": { "total_number_of_results":"" } },
                    variant_data_key: { "info_features": { "AF":"" } }
                  }

extract_keys_list = config["VCF"]["EXTRACT_KEYS"].split(" ")
extract_keys = {}
extract_keys[uta_adapter_genetogenomic_srv_prefix] = ["hgnc_symbol","aminoacid_exchange"]
extract_keys[uta_adapter_srv_prefix] = ["gene_name","variant_exchange"]
extract_keys[revel_srv_prefix] = ["Score"]
extract_keys[dbnsfp_srv_prefix] = ["SIFT_pred"]
extract_keys[vuspredict_srv_prefix] = ["FATHMM","Missense3D","SIFT","Score"]
extract_keys[dbsnp_srv_prefix] = ["rsID", "freq_total"]

for assign in extract_keys_list:
    key,vals = assign.split(":")
    extract_keys[key] = vals.split(",")

# TSV writer feature keys
#tsv_columns = [ civic_srv_prefix, oncokb_srv_prefix, metakb_srv_prefix, revel_srv_prefix, loftool_srv_prefix,
#                vuspredict_srv_prefix, dbsnp_srv_prefix, clinvar_srv_prefix, mvp_srv_prefix,
#                dbsnp_srv_prefix, loftool_srv_prefix, uta_adapter_srv_prefix, uta_adapter_genetogenomic_srv_prefix ]

#tsv_columns = [ civic_srv_prefix, oncokb_srv_prefix, metakb_srv_prefix, revel_srv_prefix, loftool_srv_prefix,
#                vuspredict_srv_prefix, dbsnp_srv_prefix, clinvar_srv_prefix, mvp_srv_prefix,
#                dbsnp_srv_prefix, loftool_srv_prefix, uta_adapter_srv_prefix, uta_adapter_genetogenomic_srv_prefix ]
tsv_mappings = {
                 "variant_data": ["CHROM","POS_hg38","POS_hg19","REF","ALT","type","blosum62"],
                 uta_adapter_srv_prefix: ["gene_name", "variant_exchange", "transcript", "variant_exchange_long"],
                 uta_adapter_genetogenomic_srv_prefix: ["c_dna_string","cds_start","cds_end","strand","prot_location"],
                 uta_adapter_protein_sequence_srv_prefix: ["protein_id", "protein_sequence"],
                 dbnsfp_srv_prefix: dbnsfp_keys,
                 revel_srv_prefix: ["Score"],
                 alphamissense_srv_prefix: ["score", "uniprot_id", "alphamissense_class"],
                 mvp_srv_prefix: ["Score"],
                 clinvar_srv_prefix: ["CLNSIG","CLNREVSTAT","CLNDN","CLINVARID"],
                 dbsnp_srv_prefix: ["freq_total", "rsID"],
                 loftool_srv_prefix: ["Score"],
                 vuspredict_srv_prefix: ["Missense3D"],
                 "protein_features": protein_features_keys,
                molecular_features_srv_prefix: molecular_features_keys,
                dgidb_srv_prefix: dgidb_keys,
                gencode_cna_srv_prefix: gencode_cna_keys
                 }
vcf_mappings = tsv_mappings
#vcf_mappings = {
#                 uta_adapter_srv_prefix: ["gene_name", "variant_exchange", "transcript", "variant_exchange_long"],
#                 uta_adapter_genetogenomic_srv_prefix: ["c_dna_string","cds_start","cds_end","strand","prot_location"],
#                 uta_adapter_protein_sequence_srv_prefix: ["protein_id", "protein_sequence"],
#                 dbnsfp_srv_prefix: dbnsfp_keys,
#                 revel_srv_prefix: ["Score"],
#                 alphamissense_srv_prefix: ["score", "uniprot_id", "alphamissense_class"],
#                 mvp_srv_prefix: ["Score"],
#                 clinvar_srv_prefix: ["CLNSIG","CLNREVSTAT","CLNDN","CLINVARID"],
#                 dbsnp_srv_prefix: ["freq_total", "rsID"],
#                 loftool_srv_prefix: ["Score"],
#                 vuspredict_srv_prefix: ["Missense3D"],
#                 "protein_features": ["DSSP","RSA"],
#                 "variant_data": ["blosum62"]
#                 }

tsv_feature_ranking = ['genomic_location_hg38', 'chrom', 'pos_hg38', 'pos_hg19', 'ref', 'alt', 'mutation_type',
                       'hgnc_gene_symbol', 'aa_exchange', 'aa_exchange_long', 'ncbi_transcript_mane_select', 'ncbi_cdna_string',
                       'ncbi_cds_start', 'ncbi_cds_end', 'ncbi_cds_strand', 'ncbi_prot_location', 'ncbi_protein_id', 'clinvar_clinical_significance',
                       'clinvar_review_status', 'clinvar_cancer_type', 'clinvar_id', 'dbsnp_population_frequency', 'dbsnp_rsid',
                       'gnomAD_exomes_ac', 'gnomAD_exomes_af', '1000genomes_af', '1000genomes_ac', 'alfa_total_af', 'alfa_total_ac',
                       'ExAC_AF', 'ExAC_AC', 'revel_score', 'alphamissense_score', 'mvp_score', 'loftool_score', 'vuspredict_score',
                       'missense3D_pred', 'SIFT_score', 'SIFT_pred', 'GERP++_score', 'MetaLR_score', 'MetaSVM_score', 'phastCons17way_primate_score',
                       'phyloP17way_primate', 'MutationAssessor_score', 'MutationTaster_score', 'fathmm-MKL_coding_score', 'fathmm-XF_coding_score',
                       'uniprot_id', 'alphamissense_class', 'Interpro_domain', 'protein_sequence_MANE_Select',
                       'Secondary_protein_structure', 'RelASA', 'BLOSUM62']

tsv_labels = {
    "mutation_type":"variant_data_type",
    "genomic_location_hg38":"qid",
    "chrom":"variant_data_CHROM",
    "pos_hg38":"variant_data_POS_hg38",
    "pos_hg19":"variant_data_POS_hg19",
    "ref":"variant_data_REF",
    "alt":"variant_data_ALT",
    "hgnc_gene_symbol": "UTA_Adapter_gene_name",
    "aa_exchange": "UTA_Adapter_variant_exchange",
    "aa_exchange_long": "UTA_Adapter_variant_exchange_long",
    "ncbi_transcript_mane_select": "UTA_Adapter_transcript",
    "ncbi_cdna_string": "UTA_Adapter_gene_c_dna_string",
    "ncbi_cds_start": "UTA_Adapter_gene_cds_start",
    "ncbi_cds_end": "UTA_Adapter_gene_cds_end",
    "ncbi_cds_strand": "UTA_Adapter_gene_strand",
    "ncbi_prot_location": "UTA_Adapter_gene_prot_location",
    "ncbi_protein_id": "UTA_Adapter_protein_sequence_protein_id",
    "clinvar_clinical_significance": "clinvar_CLNSIG",
    "clinvar_review_status": "clinvar_CLNREVSTAT",
    "clinvar_cancer_type": "clinvar_CLNDN",
    "clinvar_id": "clinvar_CLINVARID",
    "dbsnp_population_frequency": "dbsnp_freq_total",
    "dbsnp_rsid": "dbsnp_rsID",
    "gnomAD_exomes_ac": "dbnsfp_gnomAD_exomes_AC",
    "gnomAD_exomes_af": "dbnsfp_gnomAD_exomes_AF",
    "1000genomes_af": "dbnsfp_1000Gp3_AF",
    "1000genomes_ac": "dbnsfp_1000Gp3_AC",
    "alfa_total_af": "dbnsfp_ALFA_Total_AF",
    "alfa_total_ac": "dbnsfp_ALFA_Total_AC",
    "ExAC_AF": "dbnsfp_ExAC_AF",
    "ExAC_AC": "dbnsfp_ExAC_AC",
    "revel_score": "revel_Score",
    "alphamissense_score": "alphamissense_score",
    "mvp_score": "mvp_Score",
    "loftool_score": "loftool_Score",
    "vuspredict_score": "vus_predict_Score",
    "missense3D_pred": "vus_predict_Missense3D",
    "SIFT_score": "dbnsfp_SIFT_score_aggregated_value",
    "SIFT_pred": "dbnsfp_SIFT_pred_formatted",
    "GERP++_score": "dbnsfp_GERP++_RS",
    "MetaLR_score": "dbnsfp_MetaLR_score",
    "MetaSVM_score": "dbnsfp_MetaSVM_score",
    "phastCons17way_primate_score": "dbnsfp_phastCons17way_primate",
    "phyloP17way_primate": "dbnsfp_phyloP17way_primate",
    "MutationAssessor_score": "dbnsfp_MutationAssessor_score_aggregated_value",
    "MutationTaster_score": "dbnsfp_MutationTaster_score_aggregated_value",
    "fathmm-MKL_coding_score": "dbnsfp_fathmm-MKL_coding_score",
    "fathmm-XF_coding_score": "dbnsfp_fathmm-XF_coding_score",
    "uniprot_id": "alphamissense_uniprot_id",
    "alphamissense_class": "alphamissense_alphamissense_class",
    "Interpro_domain": "dbnsfp_Interpro_domain",
    "protein_sequence_MANE_Select": "UTA_Adapter_protein_sequence_protein_sequence",
    "Secondary_protein_structure": "protein_features_DSSP",
    "RelASA": "protein_features_RSA",
    "BLOSUM62": "variant_data_blosum62"
}

clinical_evidence_mappings = {
    "variant_data": ["CHROM","POS_hg38","POS_hg19","REF","ALT"],
    "onkopus_aggregator": ["merged_evidence_data"]
}

normalized_treatment_features = [ 'gene', 'variant', 'drugs', 'evidence_level', 'evidence_type', 'citation_id', 'reference', 'source' ]

clinical_evidence_match_types = ["exact_match"]
# Match types of clinical evidence data
match_types = ["exact_match","any_mutation_in_gene","same_position","same_position_any_mutation"]



import traceback, copy
import adagenes.tools.module_requests as req
from adagenes.tools.module_requests import generate_variant_dictionary
from onkopus.conf import read_config as config
import adagenes.tools.parse_genomic_data
import adagenes as ag
import onkopus as op


def strip_long_indels(variants):
    variants_new = []
    for variant in variants:
        if not len(variant) > 30:
            variants_new.append(variant)

    return variants_new

def validate_variants(variants):
    variants_new = []
    for var in variants:
        req= ag.get_variant_request_type(var)
        #print("req type: ",req)
        #if (req[0] == "genomic_location")\
        #        or (req[0] == "insertion") \
        #        or (req[0] == "deletion") \
        #        :
        if (req[0] == "genomic_location"):
            variants_new.append(var)
    return variants_new

class UTAAdapterClient:

    def __init__(self, genome_version, error_logfile=None):
        self.genome_version = genome_version
        self.info_lines= config.uta_adapter_info_lines
        self.url_pattern = config.uta_adapter_src
        self.srv_prefix = config.uta_adapter_srv_prefix
        self.genomic_keys = config.uta_genomic_keys
        self.gene_keys = config.uta_gene_keys
        self.gene_response_keys = config.uta_gene_response_keys
        self.extract_keys = config.uta_genomic_keys
        self.key_labels = config.uta_genomic_key_labels

        self.qid_key = "q_id"
        if (self.genome_version == "hg19") or (self.genome_version == "GRCh37"):
            self.qid_key = "q_id_hg19"

    def generate_variant_list(self, variants, variant_list_without_alternate_alleles):
        """
        Generates a list of variants for module requests and the associated reference genome. Prioritizes hg38/GRCh38 positions if data is available.

        :param variants:
        :param variant_list_without_alternate_alleles:
        :return:
        """
        request_genome = self.genome_version
        check_for_hg38 = False
        variant_list = []
        if (self.genome_version == "hg19") or (self.genome_version=="GRCh37"):
            check_for_hg38 = True
            request_genome="hg38"
        for var in variant_list_without_alternate_alleles:
            if check_for_hg38:
                if "POS_hg38" in variants[var]["variant_data"]:
                    hg38_id = variants[var]["variant_data"]["CHROM"] + ":" + variants[var]["variant_data"]["POS_hg38"] \
                              + variants[var]["variant_data"]["REF"] + ">" + variants[var]["variant_data"]["ALT"]
                    variant_list.append(hg38_id)
                else:
                    variant_list.append("")
            else:
                variant_list.append(var)

        return variant_list, request_genome

    def process_data(self, vcf_lines):
        """
        Retrieves genomic, transcriptomic and proteomic data from the Coordinates Converter service

        :param vcf_lines:
        :return:
        """
        #print("uta nnotate ",self.genome_version)

        # Filtering
        vcf_linesf = adagenes.tools.filter_wildtype_variants(vcf_lines)

        qid_dc = {}
        if self.genome_version == "hg38":
            qid_list = copy.deepcopy(list(vcf_linesf.keys()))
        else:
            #print("hg19 qid list ",vcf_linesf)
            qid_dc, qid_list = ag.tools.generate_qid_list_from_other_reference_genome(vcf_linesf)
            #self.genome_version = "hg38"
            retransform = True
        #qid_dc, qid_list = ag.tools.generate_qid_list_from_other_reference_genome(vcf_linesf)
        #self.genome_version = "hg38"
        #print("uta qid list ",qid_list)

        while True:
            max_length = int(config.config["DEFAULT"]["MODULE_BATCH_SIZE"])
            if max_length > len(qid_list):
                max_length = len(qid_list)
            qids_partial = qid_list[0:max_length]

            variants_without_alternate_alleles = adagenes.tools.filter_alternate_alleles(qids_partial)
            request_genome_version = self.genome_version

            variants_without_alternate_alleles = strip_long_indels(variants_without_alternate_alleles)
            variants_without_alternate_alleles = validate_variants(variants_without_alternate_alleles)
            #print("VARS",variants_without_alternate_alleles)

            if len(variants_without_alternate_alleles) > 0:
                variants = ','.join(variants_without_alternate_alleles)

                try:
                    #json_body = req.get_connection(variants, self.url_pattern, request_genome_version)
                    json_body = req.get_connection(variants, self.url_pattern, "hg38")

                    for item in json_body:
                                qid_q = str(item["header"]["qid"])

                                if qid_q in qid_dc.keys():
                                    qid = qid_dc[qid_q]
                                else:
                                    qid = qid_q

                                if qid not in vcf_lines.keys():
                                    qid = qid.replace("g.","")
                                    qid = qid.replace("p.","")
                                    qid = qid.replace("c.","")

                                if item["data"] is not None:

                                    # add variant data
                                    if config.variant_data_key not in vcf_lines[qid]:
                                        vcf_lines[qid][config.variant_data_key] = {}

                                    if type(item["data"]) is dict:

                                        if "gene name" in item["data"]:
                                            vcf_lines[qid][config.variant_data_key]['Gene name'] = item["data"]["gene_name"]
                                            vcf_lines[qid][config.variant_data_key]['Variant exchange'] = item["data"]["variant_exchange"]
                                            vcf_lines[qid][config.variant_data_key]['Biomarker'] = item["data"]["gene_name"] + " " + item["data"][
                                                "variant_exchange"]

                                        chr, ref_seq, pos, ref, alt = adagenes.tools.parse_genomic_data.parse_genome_position(
                                            qid)
                                        vcf_lines[qid][config.variant_data_key]['CHROM'] = chr
                                        vcf_lines[qid][config.variant_data_key]['reference_sequence'] = ref_seq
                                        vcf_lines[qid][config.variant_data_key]['POS'] = pos
                                        vcf_lines[qid][config.variant_data_key]['REF'] = ref
                                        vcf_lines[qid][config.variant_data_key]['ALT'] = alt
                                        vcf_lines[qid][config.variant_data_key]['POS_' + self.genome_version] = pos
                                        vcf_lines[qid][config.variant_data_key]['ID'] = ''
                                        vcf_lines[qid][config.variant_data_key]['QUAL'] = ''
                                        vcf_lines[qid][config.variant_data_key]['FILTER'] = ''

                                        vcf_lines[qid][self.srv_prefix] = item["data"]

                                    else:
                                        vcf_lines[qid][self.srv_prefix] = {}
                                        vcf_lines[qid][self.srv_prefix]["status"] = 400
                                        vcf_lines[qid][self.srv_prefix]["msg"] = item["data"]

                                    # Generate existing variant data
                except:
                    print("error: genomic to gene")
                    print(traceback.format_exc())

            for i in range(0, max_length):
                del qid_list[0] #qid_list.remove(qid)
            if len(qid_list) == 0:
                break

        return vcf_lines



import unittest
import os
import onkopus as op
import adagenes as ag


class TestCNAAnnotation(unittest.TestCase):

    def test_cna_annotation(self):
        cna_id = "chr15:30103918-34000000_DEL"
        data = { cna_id:{ "variant_data": { "CHROM": 15, "POS":30103918, "POS2": 30644082 },"mutation_type": "cnv" }}
        bframe = ag.BiomarkerFrame(data)
        #print(bframe.data)
        data = op.annotate_cnas(bframe.data)
        #print(data)
        self.assertEqual(set(list(data[cna_id].keys())),
                         {'type', 'gencode_cna', 'mdesc', 'variant_data', 'mutation_type','protein_domains', 'dgidb',
                          'cna_genes','cnvoyant', 'xcnv','isv','tada','dbcnv'},
                         "")
        #self.assertEqual(len(data[cna_id]["gencode_cna"]["cds"]),128,"")
        self.assertEqual(len(data[cna_id]["cna_genes"]),22,"")

    def test_cna_annotation_file(self):
        __location__ = os.path.realpath(
            os.path.join(os.getcwd(), os.path.dirname(__file__)))
        infile = __location__ + "/../test_files/cnv_sample.vcf"
        bframe = op.read_file(infile, genome_version="hg38")
        print("bframe keys ",bframe.data.keys())
        #data = {"chr15:30103918><DEL>": {"variant_data": {"CHROM": 15, "POS": 30103918, "POS2": 30644082},
        #                                 "mutation_type": "cnv"}}
        #bframe = ag.BiomarkerFrame(data)
        #print(bframe.data)
        data = op.annotate_cnas(bframe.data)
        #print("annotated data ", data)

        cna_id = "chr1:844347-6477436_DEL"
        self.assertEqual(set(list(data[cna_id].keys())),
                         {'type', 'gencode_cna', 'mdesc', 'variant_data', 'mutation_type', 'protein_domains',
                          'info_features','orig_identifier', 'dgidb', 'cna_genes','cnvoyant', 'xcnv','isv','dbcnv',
                          'tada'},
                         "")
        self.assertEqual(len(data[cna_id]["gencode_cna"]["cds"]), 2, "")
        self.assertEqual(data[cna_id]["variant_data"]["POS2"], "714338", "")



import unittest
import os, time
import onkopus as op
import adagenes as ag


class TestFullAnnotation(unittest.TestCase):

    def test_full_annotation(self):
        start = time.time()
        __location__ = os.path.realpath(
            os.path.join(os.getcwd(), os.path.dirname(__file__)))
        infile = __location__ + "/../test_files/somaticMutations.vcf"
        outfile = infile + ".anno.vcf"
        bframe = op.read_file(infile, genome_version="hg19")
        #bframe.data = onkopus.annotate_variant_data(bframe.data, genome_version="hg19", include_clinical_data=False)
        bframe = op.annotate(bframe)
        print(bframe.data.keys())
        op.write_file(outfile, bframe)
        stop = time.time() - start
        print("Required time: ", stop)

        with open(outfile, 'r') as file:
            read_content = file.read()

        content_exp = ('ILTER\tINFO\n'
 'chr7\t21744593\t0\tA\tAG\t0\t0\t'
 '0;chrom=chr7;pos_hg38=21744593;pos_hg19=21784211;ref=A;alt=AG\n'
 'ch')

        self.assertEqual(read_content[50:150], content_exp, "")


    def test_full_annotation_gene_and_cnv(self):
        #data = {"TP53":{ }, "chr7:140753336A>T": {}, "NRAS:Q61L": {} }
        #bframe = adagenes.BiomarkerFrame()
        __location__ = os.path.realpath(
            os.path.join(os.getcwd(), os.path.dirname(__file__)))
        #bframe = op.read_file(__location__ + "/../test_files/variants_grch38_20240206_Pathogenic_80.tsv")
        bframe = ag.BiomarkerFrame({"TP53":{},
                                    "chr1:258946-5532775_DEL":{"variant_data":{
                                        "CHROM":"1",
                                        "POS": "258946",
                                        "POS2":5532775}}
                                    })
        bframe.genome_version="hg38"
        #bframe.data = data
        #print(bframe.data)

        bframe = op.annotate(bframe)

        set0 = set(list(bframe.data["TP53"].keys()))
        set1 = {'type', 'mutation_type', 'mdesc', 'cosmic', 'civic', 'dgidb', 'gencode', 'onkopus_aggregator',
                'UTA_Adapter','UTA_Adapter_protein_sequence'}
        self.assertEqual(set0, set1, "")

        #print(bframe.data["chr1:258946A><DEL>"].keys())
        set0 = set(list(bframe.data["chr1:258946-5532775_DEL"].keys()))
        set1 = {'cna_genes','type', 'mutation_type', 'mdesc', 'dgidb', 'gencode_cna', 'variant_data', 'protein_domains',
                'cnvoyant','isv','tada','xcnv','dbcnv'}
        self.assertEqual(set0, set1, "")





tag = "main"

available_modules = ["alphamissense","clinvar","dbsnp","seqcat","dbnsfp","gencode","civic","oncokb",
                     "metakb","protein_analysis","molecular_features","drug_classification",
                     "onkopus-aggregator","onkopus-interpreter","dgidb", "gtex", "onkopus-cache",
                     "onkopus-server", "onkopus-web", "onkopus-database"]

module_ids = {
    "alphamissense": ["alphamissense.yml"],
    "dbsnp": ["dbsnp.yml"],
    "clinvar": ["clinvar.yml"],
    "dgidb": ["dgidb-db.yml", "dgidb-adapter.yml"],
    "gtex": ["gtex-adapter.yml"],
    "fathmm": ["fathmm-adapter.yml"],
    "m3d": ["m3dapp.yml"],
    "uniprot": ["uniprot-adapter.yml"],
    "vep": ["vep-adapter.yml"],
    "vus-predict": ["vus-predict.py"],
    "revel": ["revel.yml"],
    "loftool": ["loftool.yml"],
    "onkopus-server": ["onkopus-server.yml"],
    "onkopus-web": ["onkopus-web.yml"],
    "metakb": ["metakb-adapter.yml"],
    "oncokb": ["oncokb.yml"],
    "civic": ["civic-db.yml", "civic-adapter.yml"],
    "dbnsfp": ["dbnsfp-adapter.yml"],
    "primateai": ["primateai-adapter.yml"],
    "onkopus-database": ["onkopus-database.yml"],
    "onkopus-websocket-server": ["onkopus-server.yml"],
    "onkopus-cache":["onkopus-cache.yml"],
    "protein-analysis":["protein-analysis.yml"],
    "molecular-features":["molecular-features.yml"],
    "onkopus-aggregator":["onkopus-aggregator.yml"],
    "onkopus-interpreter":["onkopus-interpreter.yml"],
    "gencode":["gencode-adapter.yml", "gencode-db.yml"],
    "seqcat": ["uta-adapter.yml", "uta-database.yml"]
}



import unittest
import os, time
import onkopus as op
import adagenes as ag


class TestFullAnnotationSpeed(unittest.TestCase):

    def test_full_annotation_speed(self):
        start = time.time()
        __location__ = os.path.realpath(
            os.path.join(os.getcwd(), os.path.dirname(__file__)))
        infile = __location__ + "/../test_files/somaticMutations.hg38.ln200.vcf"
        outfile = infile + ".anno.vcf"
        bframe = op.read_file(infile, genome_version="hg38")
        print("Variants ",len(list(bframe.data.keys())))
        #bframe.data = onkopus.annotate_variant_data(bframe.data, genome_version="hg19", include_clinical_data=False)
        bframe = op.annotate(bframe, )
        print(bframe.data.keys())
        op.write_file(outfile, bframe)
        stop = time.time() - start
        print("Required time: ", stop)

        with open(outfile, 'r') as file:
            read_content = file.read()

        content_exp = ('ILTER\tINFO\n'
 'chr7\t21744593\t0\tA\tAG\t0\t0\t'
 '0;chrom=chr7;pos_hg38=21744593;pos_hg19=21784211;ref=A;alt=AG\n'
 'ch')

        #self.assertEqual(read_content[50:150], content_exp, "")


import unittest, copy, os
import onkopus as op
import adagenes as ag

def file_len(fname):
    with open(fname) as f:
        for i, l in enumerate(f):
            pass
    return i + 1

class DBSNPAnnotationTestCase(unittest.TestCase):

    def test_dbsnp_client(self):
        genome_version = 'hg38'

        data = {"chr7:140753336A>T": {}, "chr1:2556710C>A": {},
                "chr1:2556710C>T":{},"chr1:2556714A>G":{}, "chr1:2556718C>T":{},
                "chr1:2556718C>.": {}, "chr1:2556710C>.":{}
                }

        variant_data = op.DBSNPClient(
            genome_version=genome_version).process_data(data)

        print("Response ",variant_data)
        self.assertListEqual(["chr7:140753336A>T", "chr1:2556710C>A",
                "chr1:2556710C>T","chr1:2556714A>G", "chr1:2556718C>T",
                "chr1:2556718C>.", "chr1:2556710C>."], list(variant_data.keys()),
                             "")
        self.assertEqual('0:23038(0.0)', variant_data["chr1:2556710C>T"]["dbsnp"]["freq_total"], "")

    def test_dbsnp_file(self):
        __location__ = os.path.realpath(
            os.path.join(os.getcwd(), os.path.dirname(__file__)))
        infile = __location__ + "/../../test_files/somaticMutations.hg38.ln200.vcf"
        outfile = __location__ + "/../../test_files/somaticMutations.hg38.ln200.dbsnp.vcf"

        # filter_data = ['pos', {'filterType': 'number', 'type': 'greaterThan', 'filter': 47742809}]
        filter_data = "dbsnp_total > 0.8"

        client = op.DBSNPClient(genome_version="hg38")

        ag.process_file(infile, outfile, client)

        with open(outfile, 'r') as file:
            actual_contents = file.read()

        expected_contents = ''

        # Compare the actual contents to the expected contents
        # self.assertEqual(actual_contents, expected_contents,
        #                 f"File contents do not match. Expected: {expected_contents}, Actual: {actual_contents}")
        self.assertEqual(file_len(outfile), 213)



import unittest, os
import onkopus as op


class TestAnnotatedDataReader(unittest.TestCase):

    def test_read_annotated_data(self):
        __location__ = os.path.realpath(
            os.path.join(os.getcwd(), os.path.dirname(__file__)))
        infile = __location__ + "/../test_files/somaticMutations.ln50.avf"
        bframe = op.read_file(infile)




import unittest, os
import onkopus as op
import adagenes as ag

class CSVAnnotationtestCase(unittest.TestCase):

    def test_annotate_with_additional_columns(self):
        __location__ = os.path.realpath(
            os.path.join(os.getcwd(), os.path.dirname(__file__)))
        infile = __location__ + "/../test_files/somaticMutations.revel.csv"
        bframe = op.read_file(infile, genome_version="hg38")
        #print("pre features ",bframe.preexisting_features)
        outfile = __location__ + "/../test_files/somaticMutations.revel.am.esm1b.csv"

        bframe.data = op.REVELClient(genome_version="hg38").process_data(bframe.data)
        bframe.data = op.DBNSFPClient(genome_version="hg38").process_data(bframe.data)

        for var in bframe.data.keys():
            if "AlphaMissense_score_aggregated_value" in bframe.data[var]["dbnsfp"].keys():
                print("found")

        op.write_file(outfile,bframe,export_features={"AlphaMissense":"dbnsfp>AlphaMissense_score_aggregated_value","REVEL":"revel>Score"})



import copy, multiprocessing
import concurrent.futures
import traceback

import adagenes
import onkopus as op


def annotate(bframe:adagenes.BiomarkerFrame,
             genome_version=None,
             oncokb_key=None,
             lo_hg19=None,
             lo_hg38=None,
             lo_t2t=None,
             tumor_type=None,
             request_id=None,
             redis_client=None,
             protein_to_genomic=True,
             transcript_to_genomic=True,
             include_clinical_data=True,
             include_gene=True,
             include_acmg=True
             ) -> adagenes.BiomarkerFrame:
    """
    Runs the full Onkopus annotation pipeline to annotate biomarkers

    :param tumor_type:
    :param lo_t2t:
    :param lo_hg38:
    :param lo_hg19:
    :param oncokb_key:
    :param genome_version:
    :param bframe:
    :return:
    """
    if genome_version is not None:
        bframe.genome_version = genome_version

    if isinstance(bframe,dict):
        bframe = adagenes.BiomarkerFrame(data=bframe)

    # Recognize biomarkers
    bframe = adagenes.recognize_biomarker_types(bframe)

    # Map biomarkers on protein and transcript level to genomic level (MANE Select)
    if protein_to_genomic is True:
        bframe = op.ProteinToGenomic().process_data(bframe)
        bframe = op.TranscriptToGenomic().process_data(bframe)

    # Liftover
    target_genome = None
    if (bframe.genome_version != "hg38") and (bframe.genome_version != ""):
        target_genome = "hg38"
        #bframe = adagenes.LiftoverAnnotationClient(bframe.genome_version, target_genome="hg38").process_data(bframe)
        bframe = adagenes.LiftoverClient(bframe.genome_version, target_genome="hg38").process_data(bframe)
    #bframe = adagenes.liftover(bframe, target_genome=target_genome)

    # Annotate biomarkers
    #bframe.data = op.annotate_snvs(bframe.data,genome_version=bframe.genome_version,oncokb_key=oncokb_key,
    #                               lo_hg19=lo_hg19,lo_hg38=lo_hg38,lo_t2t=lo_t2t, tumor_type=tumor_type)
    #bframe.data = op.annotate_indels(bframe.data,genome_version=bframe.genome_version,oncokb_key=oncokb_key,tumor_type=tumor_type)
    #bframe.data = op.annotate_fusions(bframe.data,genome_version=bframe.genome_version,tumor_type=tumor_type)
    #bframe.data = op.annotate_genes(bframe.data, genome_version=bframe.genome_version,tumor_type=tumor_type)

    data = copy.deepcopy(bframe.data)
    num_cores = multiprocessing.cpu_count()
    if tumor_type is None:
        tumor_type = ""

    t_args = [
              { 'genome_version': bframe.genome_version ,
                'oncokb_key': oncokb_key ,
                'tumor_type': tumor_type,
                'request_id': request_id,
                'include_clinical_data': include_clinical_data,
                'include_gene': include_gene,
                'include_acmg': include_acmg
                },
              {
                  'genome_version': bframe.genome_version,
                  'oncokb_key': oncokb_key,
                  'tumor_type': tumor_type,
                  'request_id': request_id,
                  'include_clinical_data': include_clinical_data,
                  'include_gene': include_gene,
                  'include_acmg': include_acmg
              },
             {
                 'genome_version': bframe.genome_version,
                 'tumor_type': tumor_type,
                 'request_id': request_id,
                 'include_clinical_data': include_clinical_data,
                 'include_gene': include_gene,
                 'include_acmg': include_acmg
             },
             {
                 'genome_version': bframe.genome_version,
                 'tumor_type': tumor_type,
                 'request_id': request_id,
                 'include_clinical_data': include_clinical_data,
                 'include_gene': include_gene,
                 'include_acmg': include_acmg
             },
             {
                 'genome_version': bframe.genome_version,
                 'tumor_type': tumor_type,
                 'request_id': request_id,
                 'include_clinical_data': include_clinical_data,
                 'include_gene': include_gene,
                 'include_acmg': include_acmg
             }
              ]

    result_data = []
    #print("start pooling")
    with concurrent.futures.ProcessPoolExecutor() as executor:
        futures = {
            executor.submit(op.annotate_snvs, data, **t_args[0]): "snvs",
            executor.submit(op.annotate_indels, data, **t_args[1]): "indels",
            executor.submit(op.annotate_fusions, data, **t_args[2]): "fusions",
            executor.submit(op.annotate_genes, data, **t_args[3]): "genes",
            executor.submit(op.annotate_cnas, data, **t_args[4]): "cnas"
        }

        for future in concurrent.futures.as_completed(futures):
            key = futures[future]
            try:
                #print("result  ",future.result())
                result_data.append(future.result())
            except Exception as exc:
                print(f"{key} generated an exception: {exc}")
                print(traceback.format_exc())

    data_new = {}
    for result in result_data:
        #print("resultt ",result)
        data_new = adagenes.merge_dictionaries(data_new, result)
    bframe.data = data_new

    return bframe

def parallelize_annotations(data, genome_version, oncokb_key, lo_hg19, lo_hg38, lo_t2t, tumor_type, num_cores):
    annotations = []
    with multiprocessing.Pool(processes=num_cores) as pool:
        annotations.append(pool.apply_async(op.annotate_snvs, args=(data, genome_version, oncokb_key, lo_hg19, lo_hg38, lo_t2t, tumor_type)))
        annotations.append(pool.apply_async(op.annotate_indels, args=(data, genome_version, oncokb_key, tumor_type)))
        annotations.append(pool.apply_async(op.annotate_fusions, args=(data, genome_version, tumor_type)))
        annotations.append(pool.apply_async(op.annotate_genes, args=(data, genome_version, tumor_type)))
    results = [a.get() for a in annotations]
    return results




import unittest, copy, os
import onkopus as op
import adagenes as ag

class GencodeAnnotationTestCase(unittest.TestCase):

    def test_gencode_client(self):
        genome_version = 'hg19'

        data = {"chr17:7681744T>C": {}, "chr10:8115913C>T": {}}

        variant_data = op.UTAAdapterClient(genome_version=genome_version).process_data(data)
        variant_data = op.LiftOverClient(genome_version=genome_version,target_genome="hg38").process_data(variant_data)
        genome_version="hg38"
        variant_data = op.GENCODEGenomicClient(
            genome_version=genome_version).process_data(variant_data)

        print("Response ",variant_data)

    def test_gencode_client_hg38(self):
        genome_version = 'hg38'

        data = {"chr7:140753336A>T": {}}

        variant_data = op.UTAAdapterClient(genome_version=genome_version).process_data(data)
        #variant_data = onkopus.onkopus_clients.LiftOverClient(genome_version=genome_version,target_genome="hg38").process_data(variant_data)
        variant_data = op.GENCODEGenomicClient(
            genome_version=genome_version).process_data(variant_data)

        print("Response ",variant_data)

    def test_gencode_client_vcf_export(self):
        genome_version = 'hg38'
        __location__ = os.path.realpath(
            os.path.join(os.getcwd(), os.path.dirname(__file__)))
        outfile = __location__ + "/../../test_files/output_gencode_genomic.vcf"

        data = {"chr7:140753336A>T": {}}

        variant_data = op.UTAAdapterClient(genome_version=genome_version).process_data(data)
        variant_data = op.GENCODEGenomicClient(
            genome_version=genome_version).process_data(variant_data)
        op.write_file(outfile, variant_data)

        file = open(outfile)
        contents = file.read()[0:1000]
        contents_expected = """n"""
        print(contents)
        #self.assertEqual(contents, contents_expected, "")
        file.close()





import copy
from adagenes.clients import client
import onkopus as op
import adagenes
import onkopus.tools


class ProteinToGenomic(client.Client):
    """
    Transforms biomarker data on protein level to genomic level

    """

    def process_data(self, bframe):
        if isinstance(bframe, adagenes.BiomarkerFrame):
            vdata = bframe.data
        else:
            vdata = bframe

        data = op.CCSGeneToGenomicClient(genome_version=bframe.genome_version).process_data(vdata, data_type="p")
        if isinstance(data, dict):
                #new_data = {}
                #for var in data.keys():
                #    if isinstance(data[var],dict):
                #        if "UTA_Adapter_gene" in data[var].keys():
                #            if "results_string" in data[var]["UTA_Adapter_gene"]:
                #                results_string = data[var]["UTA_Adapter_gene"]["results_string"]
                #                new_data = onkopus.tools.protein_to_genomic(new_data, results_string, data[var],
                #                                                         None, genome_version=bframe.genome_version)

                if isinstance(bframe, dict):
                    bf = adagenes.BiomarkerFrame(vdata, data_type="g")
                    return bf
                else:
                    bframe.data = copy.deepcopy(data)
                    bframe.data_type = "g"
                    return bframe
        else:
                return bframe





#from adagenes.tools import generate_keys

class DefaultFeaturesClient:
    def __init__(self, genome_version, error_logfile=None):
        self.genome_version = genome_version

    def process_data(self, biomarker_data):

        #biomarker_data = generate_keys(biomarker_data)
        #return biomarker_data
        pass



import copy
import traceback, datetime, json
import urllib.parse
import adagenes as ag
from onkopus.conf import read_config as conf_reader
import requests
from json import JSONDecodeError
import onkopus as op


class ProteinDomainCNAClient:

    def __init__(self, genome_version, error_logfile=None):
        self.genome_version = genome_version
        self.info_lines= conf_reader.protein_domains_info_lines
        self.url_pattern = conf_reader.protein_domains_src
        self.srv_prefix = conf_reader.protein_domains_srv_prefix
        self.response_keys = conf_reader.protein_domains_response_keys
        self.extract_keys = conf_reader.protein_domains_keys

        self.qid_key = "q_id"
        if (self.genome_version == "hg19") or (self.genome_version == "GRCh37"):
            self.qid_key = "q_id_hg19"
        self.error_logfile = error_logfile

    def process_data(self, biomarker_data, plot=None):
            """
            Generates JSON data for a Plotly variant needleplot with added variant annotations

            :param variant_data:
            :return:
            """
            qid_gene_name_dc = {}
            qid_list=[]
            #for qid in data.keys():
            #    variant_data = data[qid]
            #    if "UTA_Adapter" in variant_data:
            #        if ("gene_name" in variant_data["UTA_Adapter"]) and ("variant_exchange" in variant_data["UTA_Adapter"]):
            #            qid_gene_name_dc[variant_data["UTA_Adapter"]["gene_name"] + ':' + variant_data["UTA_Adapter"]["variant_exchange"]] = qid
            #            qid_list.append(variant_data["UTA_Adapter"]["gene_name"] + ':' + variant_data["UTA_Adapter"]["variant_exchange"])
            #    else:
            #        #qid_list.append(qid)
            #        #print("ProteinFeatures: Could not find UTA adapter section: ", variant_data)
            #        pass

            #vcf_linesf = ag.tools.filter_wildtype_variants(biomarker_data)
            vcf_linesf = biomarker_data

            qid_dc = {}
            retransform = False
            if self.genome_version == "hg38":
                qid_list0 = copy.deepcopy(list(vcf_linesf.keys()))
            else:
                qid_dc, qid_list0 = ag.tools.generate_qid_list_from_other_reference_genome(vcf_linesf)
                self.genome_version = "hg38"
                retransform = True

            qid_list = []
            genes = []
            variants = []

            genes_cna_dc = {}
            for var in qid_list0:
                if retransform is False:
                    var_orig = var
                else:
                    qid_orig = qid_dc[var]
                    var_orig = qid_orig

                if "gencode_cna" in biomarker_data[var_orig]:
                    if "gene" in biomarker_data[var_orig]["gencode_cna"]:
                        for gene in biomarker_data[var_orig]["gencode_cna"]["gene"]:
                            gene_name = gene[" gene_name"]
                            if gene_name not in genes_cna_dc:
                                genes_cna_dc[gene_name] = []
                            genes_cna_dc[gene_name].append(var)

                            genes.append(gene_name)
                            #variants.append(biomarker_data[var_orig]["UTA_Adapter"]["variant_exchange"])
                            #qid_list.append(var)
                            qid_gene_name_dc[gene_name] = var_orig
                            qid_list.append(gene_name)
                    else:
                        pass
                elif "UTA_Adapter_gene_name" in biomarker_data[var_orig].keys():
                    genes.append(biomarker_data[var_orig]["UTA_Adapter_gene_name"])
                    variants.append(biomarker_data[var_orig]["UTA_Adapter_variant_exchange"])
                    #qid_list.append(var)
                    qid_gene_name_dc[
                        biomarker_data[var_orig]["UTA_Adapter_gene_name"] ] = var_orig
                    qid_list.append(
                        biomarker_data[var_orig]["UTA_Adapter_gene_name"] )
                elif "hgnc_gene_symbol" in biomarker_data[var_orig].keys():
                    genes.append(biomarker_data[var_orig]["hgnc_gene_symbol"])
                    variants.append(biomarker_data[var_orig]["aa_exchange"])
                    #qid_list.append(var)
                    qid_gene_name_dc[
                        biomarker_data[var_orig]["hgnc_gene_symbol"]] = var_orig
                    qid_list.append(
                        biomarker_data[var_orig]["hgnc_gene_symbol"])
                #elif "INFO" in biomarker_data[var].keys():
                #    pass
                elif "info_features" in biomarker_data[var_orig].keys():
                    #print("INFO ok")
                    #print(biomarker_data[var]["info_features"])
                    if (("UTA_Adapter_gene_name" in biomarker_data[var_orig]["info_features"]) and
                            ("UTA_Adapter_variant_exchange" in biomarker_data[var_orig]["info_features"])):
                        try:
                            genes.append(biomarker_data[var_orig]["info_features"]["UTA_Adapter_gene_name"])
                            variants.append(biomarker_data[var_orig]["info_features"]["UTA_Adapter_variant_exchange"])
                            #qid_list.append(var)
                            qid_gene_name_dc[
                                biomarker_data[var_orig]["info_features"]["UTA_Adapter_gene_name"]] = var_orig
                            qid_list.append(
                                biomarker_data[var_orig]["info_features"]["UTA_Adapter_gene_name"])
                        except:
                            print(traceback.format_exc())


            q_lists = list(op.tools.divide_list(copy.deepcopy(qid_list), chunk_size=100))
            #print("pfeatures ",q_lists)

            for q_list in q_lists:

                variants_enc = []
                for var in q_list:
                    # var_encoded = urllib.parse.quote(var, safe='=&')
                    var_encoded = urllib.parse.quote(var)
                    variants_enc.append(var_encoded)
                variants = variants_enc

                #q = ",".join(q_list)
                q = ",".join(variants)

                url = conf_reader.protein_module_domains_src + "/" + q
                print(url)
                try:
                    json_obj = requests.get(url, timeout=60).json()
                    for result in json_obj:
                        if result["data"] is not None:
                            qid = qid_gene_name_dc[result["header"]["qid"]]
                            if self.srv_prefix not in biomarker_data[qid]:
                                biomarker_data[qid][self.srv_prefix] = {}


                            gene = result["header"]["qid"]
                            biomarker_data[qid][self.srv_prefix][gene] = result["data"]
                except JSONDecodeError:
                    print("Could not decode JSON: ",traceback.format_exc())
                except KeyError:
                    print("Could not parse variant: ",traceback.format_exc())

            return biomarker_data



import datetime, traceback, copy
import adagenes.tools
import adagenes.tools.module_requests as req
from onkopus.conf import read_config as config
import adagenes.tools.parse_drug_names
from adagenes.tools import generate_variant_dictionary


def process_drug_class_labels(labels):
    """
    Turns the first drug class word to uppercase and capitalizes the second word

    :param labels:
    :return:
    """
    if labels is None:
        return labels
    if isinstance(labels, list):
        processed_labels = []
        for label in labels:
            words = label.split()
            if len(words) == 2:
                words[0] = words[0].upper()
                words[1] = words[1].capitalize()
                processed_label = ' '.join(words)
                processed_labels.append(processed_label)
            else:
                processed_labels.append(label)
        return processed_labels
    else:
        return labels


class PubChemDrugNormalizationClient:
    """
    Client for the Onkopus drug class module
    """

    def __init__(self, genome_version, error_logfile=None):
        self.genome_version = genome_version
        self.url_pattern = config.drugclass_src
        self.srv_prefix = config.drugclass_srv_prefix
        self.extract_keys = config.drugclass_keys
        self.info_lines = config.drugclass_info_lines
        self.error_logfile = error_logfile

        self.qid_key = "q_id"
        if (self.genome_version == "hg19") or (self.genome_version == "GRCh37"):
            self.qid_key = "q_id_hg19"

    def get_drug_list(self, variant_keys,variants,section="merged_evidence_data"):
        """
        Retrieves a list of all drug names found in all treatment options for all biomarkers in a biomarker set

        :param variants:
        :return:
        """
        drug_list=[]
        #print("keys ", variant_keys)
        for qid in variant_keys:
            try:
                q_drugs = ""
                therapy_list = []
                #for section in config.clinical_evidence_match_types:
                for match_type in config.match_types:
                    if match_type in variants[qid]["onkopus_aggregator"][section]:
                        for i, treatment in enumerate(variants[qid]["onkopus_aggregator"][section][match_type]):

                                therapy = []
                                # q_drugs += treatment["drugs"] + ";"

                                if "drugs" in treatment:
                                    #print("drugs ",treatment["drugs"])
                                    drugs = treatment["drugs"]
                                    for d, drug in enumerate(drugs):
                                        if isinstance(drug, dict):
                                            if "drug_name" in drug:
                                                if drug['drug_name'] != "":
                                                    drug_name = drug['drug_name'].lower()
                                                    drug_name = drug_name.replace("/","")
                                                    therapy.append(drug_name)
                                                    drug_list.append(drug_name)
                                            else:
                                                pass
                                                #print("no drug name found ", drug)
                                        elif isinstance(drug,str):
                                            drug_name = drug.lower()
                                            therapy.append(drug_name)
                                            drug_list.append(drug_name)
                                    therapy_list.append(therapy)
                                else:
                                    print("Error: No drugs section found for ",qid," for treatment ",treatment)
            except:
                print("error getting list of drugs from variants")
                print(traceback.format_exc())

        # Remove duplicates from drug list
        drug_list = list(dict.fromkeys(drug_list))

        return drug_list

    def get_drug_classes(self, drug_list):
        """
        Retrieves drug classes from the Onkopus DrugOn adapter

        :param drug_list: List of drug names
        :return: Dictionary that contains drug names as keys and a list of drug classifications as values
        """
        drug_classifications = {}

        while True:
            max_length = int(config.config["DEFAULT"]["MODULE_BATCH_SIZE"])
            if max_length > len(drug_list):
                max_length = len(drug_list)
            qids_partial = drug_list[0:max_length]

            try:
                query = ""
                for drug in qids_partial:
                    if drug not in drug_classifications:
                        query += drug + ","
                query = query.rstrip(",")
                #print(query)
                json_body = req.get_connection(query, self.url_pattern, self.genome_version)

                for result in json_body:
                    #for drug_name in json_body.keys():
                    drug_classes = result["data"]
                    drug_name = result["header"]["qid"]
                    drug_classifications[drug_name.lower()] = drug_classes
            except:
                print("Error retrieving drug classes ")
                print(traceback.format_exc())

            for i in range(0, max_length):
                del drug_list[0]
            if len(drug_list) == 0:
                break
        #print("drug classifications ",drug_classifications)
        #print(drug_classifications)
        return drug_classifications

    def process_data(self, vcf_lines, section="merged_match_types_data", match_type="exact_match"):
            """
            Retrieves drug classes for all drugs found in the aggregated, merged clinical significance data section of each biomarker

            :param vcf_lines:
            :param section:
            :param match_type:
            :return:
            """
            mut_types = False
            if (len(list(vcf_lines.keys()))==5) and ("snvs" in list(vcf_lines.keys())):
                vcf_lines_cp = copy.deepcopy(vcf_lines)
                vcf_lines = vcf_lines["snvs"]
                mut_types = True

            vcf_lines = adagenes.tools.parse_drug_names.parse_drug_names(vcf_lines,config.match_types)


            # for each variant, extract drugs from treatment results and query drug classes
            #sections = ["merged_evidence_data","merged_match_types_data"]

            qid_list = copy.deepcopy(list(vcf_lines.keys()))
            #while True:
            max_length = int(config.config["DEFAULT"]["MODULE_BATCH_SIZE"])
            if max_length > len(qid_list):
                max_length = len(qid_list)
            qids_partial = qid_list[0:max_length]


            variant_keys = adagenes.tools.filter_alternate_alleles(list(vcf_lines.keys()))
            #variant_keys = list(vcf_lines.keys())

            drug_list = self.get_drug_list(variant_keys,vcf_lines)
            #print("drug list ",drug_list)

            # Retrieve drug classifications from API endpoint
            drug_class_dc = self.get_drug_classes(copy.copy(drug_list))
            #print("drug classes ",drug_class_dc)

            for qid in variant_keys:
                    vcf_lines[qid]["drug_classes"] = drug_class_dc
                    try:
                        # merged evidence data
                        #for match_type in config.match_types:
                        if match_type in vcf_lines[qid]["onkopus_aggregator"][section]:
                                for i, treatment in enumerate(vcf_lines[qid]["onkopus_aggregator"][section][match_type]):
                                        for d,drug in enumerate(treatment["drugs"]):
                                            drug_class = ""
                                            if "drug_name_norm" in drug:
                                                drug_name = drug["drug_name_norm"]
                                                if drug_name.lower() in drug_class_dc:
                                                    drug_class = drug_class_dc[drug_name.lower()]
                                            try:
                                                if isinstance(vcf_lines[qid]["onkopus_aggregator"][section]
                                                                  [match_type][i]["drugs"][d],dict):
                                                    if "drug_class" not in vcf_lines[qid]["onkopus_aggregator"][section][match_type][i]["drugs"][d]:
                                                            vcf_lines[qid]["onkopus_aggregator"][section][match_type][i][
                                                                    "drugs"][d]["drug_class"] = []
                                                    #print("add drug class ",drug_class)
                                                    vcf_lines[qid]["onkopus_aggregator"][section][match_type][i][
                                                        "drugs"][d]["drug_class"]= process_drug_class_labels(drug_class) # .append(drug_class)
                                                else:
                                                    pass
                                                    #print("No parseable result: ",print(vcf_lines[qid]["onkopus_aggregator"][section][match_type][i][
                                                    #    "drugs"][d]))
                                            except:
                                                print("error assigning drug class")
                                                print(traceback.format_exc())

                        # merged match types data
                        for i, treatment in enumerate(vcf_lines[qid]["onkopus_aggregator"][section]):
                            for d, drug in enumerate(treatment["drugs"]):
                                drug_class = ""
                                if "drug_name" in drug:
                                    #drug_name = drug["drug_name_norm"]
                                    drug_name = drug["drug_name"]
                                    if drug_name.lower() in drug_class_dc:
                                        #print("assign drug class ",drug_name,", ",drug_class_dc[drug_name.lower()])
                                        drug_class = drug_class_dc[drug_name.lower()]
                                    else:
                                        pass
                                        #print("no dc found ",drug_name)
                                try:
                                    if isinstance(vcf_lines[qid]["onkopus_aggregator"][section][i]["drugs"][d], dict):
                                        if "drug_class" not in \
                                                vcf_lines[qid]["onkopus_aggregator"][section][i]["drugs"][d]:
                                            vcf_lines[qid]["onkopus_aggregator"][section][i]["drugs"][d]["drug_class"] = []
                                        # print("add drug class ",drug_class)

                                        vcf_lines[qid]["onkopus_aggregator"]["merged_match_types_data"][i][
                                            "drugs"][d]["drug_class"] = process_drug_class_labels(drug_class)  # .append(drug_class)
                                    else:
                                        pass
                                        #print("No parseable result: ",
                                        #      print(vcf_lines[qid]["onkopus_aggregator"][section][i]["drugs"][d]))
                                except:
                                    print("error assigning drug class")
                                    print(traceback.format_exc())

                    except:
                        if self.error_logfile is not None:
                            print("error processing request: ", vcf_lines, file=self.error_logfile + str(datetime.datetime) + '.log')
                        else:
                            print(": error processing variant response: ;", traceback.format_exc())

            if mut_types is True:
                vcf_lines_cp["snvs"] = vcf_lines
                return vcf_lines_cp

            return vcf_lines



import datetime, requests, traceback, copy
import adagenes.tools.module_requests as req
from onkopus.conf import read_config as config
import adagenes.tools
import adagenes as ag

qid_key = "q_id"
error_logfile=None


def normalize_scores(score, val):
    if score == "ESM1b_score_aggregated_value":
        score_norm = (-1) * ((float(val) - 0.0) / (25.0))
        return score_norm
    return 0


def round_raw_scores(variant_data, scores):
    """
    Rounds the score values to 3 decimal places

    :param variant_data:
    :param scores:
    :return:
    """
    for score in scores:
        if score in variant_data:
            try:
                if is_numeric_score(variant_data[score]):
                    if isinstance(variant_data[score],str):
                        if ";" in variant_data[score]:
                            max_score = get_max_value(score,variant_data[score])
                            val = round(float(max_score), 2)
                            variant_data[score+"_aggregated_value"] = val
                        elif "aggregated_value" in score:
                            val = round(float(variant_data[score]), 2)
                            variant_data[score] = val
                        else:
                            max_score = variant_data[score]
                            variant_data[score] = round(float(max_score), 2)
                    else:
                        max_score = variant_data[score]
                        variant_data[score] = max_score
                else:
                    val = variant_data[score]
                    variant_data[score] = val
            except:
                print(score,": ",traceback.format_exc())
    return variant_data


def get_max_value(score_label,score):
    values = score.split(";")
    score_max = 0.0
    # score_max = ""
    for val in values:
        if (val != "") and (val != "."):
            if is_numeric_score(val):
                if isinstance(score, str):
                    score_max = val
                else:
                    if score_label != "ESM1b":
                        if float(val) > score_max:
                            score_max = float(val)
                    else:
                        if float(val) < score_max:
                            score_max = float(val)
            else:
                return val
    return score_max


def is_numeric_score(inputString):
    if isinstance(inputString,str):
        return any(char.isdigit() for char in inputString)
    elif isinstance(inputString,float):
        return True

class DBNSFPClient:
    def __init__(self, genome_version, error_logfile=None):
        self.genome_version = genome_version
        self.info_lines = config.dbnsfp_info_lines
        self.url_pattern = config.dbnsfp_src
        self.srv_prefix = config.dbnsfp_srv_prefix
        self.extract_keys = config.dbnsfp_keys

        self.qid_key = "q_id"
        self.error_logfile = error_logfile
        #if (self.genome_version == "hg19") or (self.genome_version == "GRCh37"):
        #    self.qid_key = "q_id_hg19"

    def get_connection(self, variants, url_pattern, genome_version):
        url = url_pattern.format(genome_version) + variants
        print(url)
        r = requests.get(url)
        return r.json()

    def process_data(self, vcf_lines):
        vcf_linesf = adagenes.tools.filter_wildtype_variants(vcf_lines)

        qid_dc = {}
        if self.genome_version == "hg38":
            qid_list = copy.deepcopy(list(vcf_linesf.keys()))
        else:
            qid_dc, qid_list = ag.tools.generate_qid_list_from_other_reference_genome(vcf_linesf)
            self.genome_version = "hg38"
            retransform = True
        #qid_dc, qid_list = ag.tools.generate_qid_list_from_other_reference_genome(vcf_linesf)
        self.genome_version = "hg38"

        while True:
            max_length = int(config.config["DEFAULT"]["MODULE_BATCH_SIZE"])
            if max_length > len(qid_list):
                max_length = len(qid_list)
            qids_partial = qid_list[0:max_length]

            variants = ','.join(adagenes.tools.filter_alternate_alleles(qids_partial))

            try:
                json_body = req.get_connection(variants, self.url_pattern, self.genome_version)

                for qid, json_obj in json_body.items():
                    json_obj = json_obj["dbnsfp"]
                    if json_obj:
                        # scores to be rounded
                        scores_list = [
                            "SIFT_score_aggregated_value",
                            "fathmm-MKL_coding_score",
                            "fathmm-XF_coding_score",
                            "phastCons17way_primate_score",
                            "phyloP17way_primate_score",
                            "MetaLR_score_aggregated_value",
                            "MPC_score_aggregated_value",
                            "M-CAP_score",
                            "HUVEC_fitCons_score",
                            "Eigen-raw_coding",
                            "ClinPred_score",
                            "PROVEAN_score_aggregated_value",
                            "VEST4_score_aggregated_value",
                            "MutationTaster_score_aggregated_value",
                            "MPC_score_aggregated_value",
                            "VEST4_score_aggregated_value",
                            "Polyphen2_HDIV_score_aggregated_value",
                            "Polyphen2_HVAR_score_aggregated_value",
                            "SIFT_score_aggregated_value",
                            "MutationAssessor_score_aggregated_value",
                            "MutationTaster_score_aggregated_value",
                            "CADD_raw_aggregated_value",
                            "DANN_score",
                            "LRT_score",
                            "EVE_score",
                            "ESM1b_score",
                            "VARITY_ER_LOO_score",
                            "gMVP_score",
                            "MVP_score_aggregated_value"
                        ]

                        # calculate percentage
                        scores = ["SIFT_converted_rankscore",
                                  "Polyphen2_HDIV_rankscore",
                                  "Polyphen2_HVAR_rankscore",
                                  "fathmm-MKL_coding_rankscore",
                                  "fathmm-XF_coding_rankscore",
                                  "phastCons17way_primate_rankscore",
                                  "phyloP17way_primate_rankscore",
                                  "MetaLR_rankscore",
                                  "MPC_rankscore",
                                  "M-CAP_rankscore",
                                  "HUVEC_fitCons_rankscore",
                                  "Eigen-raw_coding_rankscore",
                                  "ClinPred_rankscore",
                                  "PROVEAN_converted_rankscore",
                                  "MutationTaster_converted_rankscore",
                                  "VEST4_rankscore",
                                  "CADD_raw_rankscore",
                                  "REVEL_rankscore",
                                  "MutationAssessor_rankscore",
                                  "PrimateAI_rankscore",
                                  "GERP++_RS_rankscore",
                                  "SIFT_converted_rankscore",
                                  "phastCons17way_primate_rankscore",
                                  "LRT_converted_rankscore",
                                  "REVEL_rankscore",
                                  "EVE_rankscore",
                                  "ESM1b_rankscore",
                                  "VARITY_ER_LOO_rankscore",
                                  "gMVP_rankscore"
                                  ]
                        for score in scores:
                            try:
                                if score in json_obj:
                                    score_percent = score + '_percent'
                                    if json_obj[score] != ".":
                                        if ";" in json_obj[score]:
                                            json_obj[score] = get_max_value(score,json_obj[score])
                                        if is_numeric_score(json_obj[score]):
                                            json_obj[score_percent] = int(float(str(json_obj[score])) * 100)
                                else:
                                    print("Could not find score in response: ",score)
                            except:
                                print("error ",score,": ", traceback.format_exc())

                        # split multiple semicolon-separated score values and select maximum value
                        scores = [
                                    "MutationTaster_score",
                                    "MPC_score",
                                    "VEST4_score",
                                    "Polyphen2_HDIV_score",
                                    "Polyphen2_HVAR_score",
                                    "SIFT_score",
                                    "MutationAssessor_score",
                                    "MutationTaster_score",
                                    "CADD_raw",
                                    "DEOGEN2_score",
                                    "FATHMM_score",
                                    "LIST-S2_score",
                                    "LRT_score",
                                    "MPC_score",
                                    "MVP_score",
                                    "MetaRNN_score",
                                    "PROVEAN_score",
                                    "REVEL_score",
                                    "SIFT4G_score",
                                    "VARITY_ER_LOO_score",
                                    "VARITY_ER_score",
                                    "VARITY_R_LOO_score",
                                    "VARITY_R_score",
                                    "gMVP_score",
                                    "VARITY_ER_LOO_score",
                                    "VARITY_ER_score",
                                    "VARITY_R_LOO_score",
                                    "VARITY_R_score",
                                    "EVE_score",
                                    "ESM1b_score",
                                    "VARITY_ER_LOO_score",
                                    "AlphaMissense_score"
                                  ]
                        min_scores = ["PROVEAN_score"]

                        # format semicolon-separated scores, max values as pathogenic
                        for score in scores:
                            try:
                                if score in json_obj:
                                    score_percent = score + '_aggregated_value'
                                    score_max = get_max_value(score,json_obj[score])
                                    json_obj[score_percent] = score_max
                                else:
                                    print("Could not find score in response: ",score,": ",json_obj)
                            except:
                                print("error ",score,": ", traceback.format_exc())

                        # format semicolon-separated scores, min values as pathogenic
                        for score in min_scores:
                            try:
                                if score in json_obj:
                                    score_percent = score + '_aggregated_value'
                                    values = json_obj[score].split(";")
                                    score_max = 0.0
                                    #score_max = ""
                                    for val in values:
                                        if (val != "") and (val != "."):
                                            if float(val) < score_max:
                                                score_max = float(val)
                                    json_obj[score_percent] = score_max
                                else:
                                    print("Could not find score in response: ",score,": ",json_obj)
                            except:
                                print("error ",score,": ", traceback.format_exc())

                        # format multiple semicolon-separated predictions (select highest value)
                        predictions = ["SIFT_pred",
                                       "Polyphen2_HDIV_pred",
                                       "Polyphen2_HVAR_pred"
                                        ]
                        for pred in predictions:
                            if pred in json_obj:
                                try:
                                    pred_formatted= pred+"_formatted"
                                    values = json_obj[pred].split(";")
                                    #score_max = 0.0
                                    score_max=""
                                    for val in values:
                                        if (val!="") and (val!="."):
                                    #        if float(val) > score_max:
                                    #            score_max = float(val)
                                            score_max = val
                                            if val == "D":
                                                score_max += " (probably damaging)"
                                            elif val == "P":
                                                score_max += " (possibly damaging)"
                                            elif val == "B":
                                                score_max += " (benign)"
                                    json_obj[pred_formatted] = score_max
                                except:
                                    print("error formatting scores: ",traceback.format_exc())
                            else:
                                print("Could not find score in response: ",pred)

                        # normalize scores
                        scores_norm = ["ESM1b_score_aggregated_value"]
                        for score in scores_norm:
                            if score in json_obj:
                                score_val_norm = normalize_scores(score, json_obj[score])
                                json_obj[score + "_normalized"] = score_val_norm

                        try:
                            json_obj = round_raw_scores(json_obj, scores_list)
                            if qid in qid_dc:
                                qid_orig = qid_dc[qid]
                            else:
                                qid_orig = qid
                            vcf_lines[qid_orig][self.srv_prefix] = json_obj
                        except:
                            print("error ",traceback.format_exc())

            except:
                print(traceback.format_exc())

            for i in range(0, max_length):
                del qid_list[0]
            if len(qid_list) == 0:
                break

        #print("return dbnsfp ",vcf_lines)
        return vcf_lines



import traceback, copy
import datetime
import adagenes.tools.module_requests as req
from onkopus.conf import read_config as config
import adagenes.tools
import adagenes as ag


class TADAClient:

    def __init__(self, genome_version, error_logfile=None):
        self.genome_version = genome_version
        self.url_pattern = config.tada_src
        self.srv_prefix = config.tada_srv_prefix
        self.extract_keys = config.tada_keys
        self.info_lines = config.tada_info_lines
        self.error_logfile = error_logfile

        self.qid_key = "q_id"
        #if (self.genome_version == "hg19") or (self.genome_version == "GRCh37"):
        #    self.qid_key = "q_id_hg19"

    def process_data(self, vcf_lines):
        #print("tada")

        # Filtering
        vcf_linesf = adagenes.tools.filter_wildtype_variants(vcf_lines)

        qid_dc = {}
        if self.genome_version == "hg38":
            qid_list = copy.deepcopy(list(vcf_linesf.keys()))
        else:
            qid_dc, qid_list = ag.tools.generate_qid_list_from_other_reference_genome(vcf_linesf)
            self.genome_version = "hg38"
            retransform = True
        #qid_dc, qid_list = ag.tools.generate_qid_list_from_other_reference_genome(vcf_linesf)
        #print("qiddc ",qid_dc)
        #self.genome_version = "hg38"

        while True:
            max_length = int(config.config["DEFAULT"]["MODULE_BATCH_SIZE"])
            if max_length > len(qid_list):
                max_length = len(qid_list)
            qids_partial = qid_list[0:max_length]
            variants = ','.join(adagenes.tools.filter_alternate_alleles(qids_partial))

            try:
                #json_body = req.get_connection(variants, self.url_pattern, self.genome_version)
                json_body = req.get_connection(variants, self.url_pattern, "hg38")

                for qid in json_body.keys():
                        try:
                            if qid in qid_dc:
                                qid_orig = qid_dc[qid]
                            else:
                                qid_orig = qid
                            #print("qid orig ",qid_orig)
                            #vcf_lines[qid_orig][self.srv_prefix] = json_obj
                            vcf_lines[qid][self.srv_prefix] = json_body[qid]
                        except:
                            cur_dt = datetime.datetime.now()
                            date_time = cur_dt.strftime("%m/%d/%Y, %H:%M:%S")
                            msg = date_time + ": Error (ClinVar client): " + ": error processing variant response: ", qid, ';', traceback.format_exc()
                            print(traceback.format_exc())


            except:
                if self.error_logfile is not None:
                    print("error processing request: ", variants, file=self.error_logfile)

            for i in range(0, max_length):
                del qid_list[0] #qid_list.remove(qid)
            if len(qid_list) == 0:
                break

        return vcf_lines
